import os,sys,json
sys.path.append(os.path.split(__file__)[0])
from .lib_collectdata.collectdata import RESULT
from .lib_collectdata.comm import get_metric_from_str
import argparse
import traceback

def parse_param(paramf):
    if os.path.isfile(paramf):
        print("Read the parameters from %s" % paramf)
        allparams = json.load(open(paramf))
        if 'PARAM' not in allparams:
            print("PARAM is not defined in %s" % paramf)
            if "post_dft" in allparams and isinstance(allparams["post_dft"],dict) and "metrics" in allparams["post_dft"] and \
                isinstance(allparams["post_dft"]["metrics"],dict) and "metrics_name" in allparams["post_dft"]["metrics"]:
                    print("Find post_dft/metrics/metrics_name, and will collectdata by these metrics")
                    return allparams["post_dft"]["metrics"]["metrics_name"]
            else:
                return []
        else:
            return allparams["PARAM"]
    else:
        print("ERROR: can not find file %s" % paramf)
        return []

def get_metric_value(result,metric_str):
    if "{" not in metric_str:
        return result[metric_str]
    else:
        metrics = get_metric_from_str(metric_str)
        values = {}
        for i in metrics:
            values[i] = result[i]
        metrics_real_str = metric_str.format(**values)
        try:
            value = eval(metrics_real_str)
        except:
            print("ERROR: can not evaluate %s" % metric_str)
            print("     The real str is: %s" % metrics_real_str)
            traceback.print_exc()
            value = None
        return value

def parse_value(abacus_result,allparams):
    allresult = {}
    for param in allparams:
        if isinstance(param,str):
            allresult[param] = get_metric_value(abacus_result,param)
        elif isinstance(param,dict):
            # if is a dict, then the key is a rename of param, and the value is an eval string, and the key of abacus_result can used by {key}
            for k,v in param.items():
                if not isinstance(k,str):
                    print("Error: %s should be a string, skip!" % str(k))
                    continue
                
                # remain when key is INPUT, indicate want to get the sub value of INPUT that is a dict,
                # such as INPUT["ecutwfc"], INPUT["basis"]
                # this can be achieved by type "{INPUT}['ecutwfc']"
                
                if k in ["INPUT"]:
                    value = abacus_result[k]
                    if not isinstance(value,dict):
                        print("the value of %s is not a dictionary" % k)
                        allresult[k] = value
                    else:
                        if isinstance(v,str):
                           allresult["%s/%s"% (k,v)] = value.get(v,None)
                        elif isinstance(v,(list,tuple)):
                            for iv in v:
                                allresult["%s/%s"% (k,iv)] = value.get(iv,None)
                        else:
                            print("%s should be str, list or tuple" % str(v))
                else:
                    if isinstance(v,str):
                        allresult[k] = get_metric_value(abacus_result,v)
                    else:
                        print("%s should be str" % str(v))
        else:
            print("%s should be str or dict" % str(param))
    return allresult                        

def CollectDataArgs(parser):
    parser.description = "This script is used to collect some key values from the output of ABACUS/QE/VASP jobs"
    parser.add_argument('-j', '--jobs', default=["."], help='the path of jobs', action="extend",nargs="*")
    parser.add_argument('-t', '--type', type=int, default=0, help='0:abacus, 1:qe, 2:vasp. Default: 0',choices=[0,1,2])
    parser.add_argument('-p', '--param', type=str, default=None, help='the parameter file, should be .json type')
    parser.add_argument('-o', '--output', type=str, default="metrics.json",help='the file name to store the output results, default is "metrics.json"')
    parser.add_argument('-m', '--modules',help='add extra modules. Default only module \'job-type\' will be loaded, such as: \'abacus\' for abacus type. You can check all modules by --outparam', action="extend",nargs="*")
    parser.add_argument('--newmethods', help='the self-defined python modules, and shuold be format of import, such as "abc"(the file name is abc.py), "a.b.c" (teh file is a/b/c.py)', action="extend",nargs="*")
    parser.add_argument('--outparam', nargs='?',type=int, const=1, default=0,help='output the registed parameters, you can set the type by -t or --type to choose abacus/qe/vasp. 0: No, 1: yes')
    parser.add_argument('--ref', type=str, nargs='?',default=None,const="resultREF.json",help='A json file includes the reference value of some keys. Generally, get values of keys start with \"delta_\" require this file. Default is resultREF.json')
    return parser

NO_PARAM_WARNING = """
WARNING: you have not defined the parameters to collect.
         You can specify by -p or --param. The context can be like: 
         {
             "PARAM":["natom","total_time"]
         }
         Only collect some common parameters, such as natom, total_time, converge, normal_end, etc.
         
         You can execute below commnd to get all the parameters that can be collected:
             abacustest collectdata --outparam
"""

def collectdata(param):    
    outputf = param.output
    paramf = param.param
    alljobs = param.jobs if len(param.jobs) == 1 else param.jobs[1:]

    alltype = {0:"abacus",1:"qe",2:"vasp"}
    jobtype = alltype.get(param.type)
    
    if param.outparam:
        RESULT(fmt=jobtype,outparam=True,newmethods=param.newmethods,modules=param.modules)
        return   
    if paramf == None:
        allparams = []
    elif not os.path.isfile(paramf):
        print("ERROR: can not find parameter file %s!!!" % paramf)
        return
    else:
        allparams = parse_param(paramf)
        
    allresult = {}
    for ipath in alljobs:
        if not os.path.isdir(ipath):
            print("ERROR: %s is not a directory, skip it!" % ipath)
            continue

        print("Handle %s" % ipath)
    
        result = RESULT(fmt=jobtype, path=ipath,newmethods=param.newmethods,modules=param.modules,resultREF=param.ref)
        if len(allparams) == 0:
            print(NO_PARAM_WARNING)
            allparams = ["normal_end","converge","nkstot","ibzk",
                "nbands","nelec","natom","scf_steps","total_time",
                "scf_time",
                {
                    "scf_time/step": "{scf_time}/{scf_steps}",
                    "ks_solver": "{INPUT}['ks_solver']"
                },
                "stress_time",
                "energy",
                "energy_per_atom",
                "force",
                "stress",
                "drho_last",
                "denergy_last",
                "version"]
            #allparams = list(result.AllMethod().keys())
        allresult[ipath] = parse_value(result,allparams)

    print("Write the results to %s" % outputf)
    json.dump(allresult,open(outputf,"w"),indent=4)
    
    from .outresult import pandas_out
    pandas_out(allresult)

def main():
    parser = argparse.ArgumentParser()
    param = CollectDataArgs(parser).parse_args()
    collectdata(param)
    
if __name__ == "__main__":
    main()






from ..model import Model
import os, glob, json, traceback, copy
from . import comm
import numpy as np
from abacustest.lib_prepare.abacus import ReadInput, WriteInput, ReadKpt, WriteKpt, AbacusStru
from abacustest.constant import RY2EV
from abacustest.lib_collectdata.comm import cal_band_gap
from abacustest.constant import RECOMMAND_IMAGE

class BandModel(Model):
    @staticmethod
    def model_name(): # type: ignore
        '''
        Name of the model, which will be used as the subcommand
        '''
        return "band"
    
    @staticmethod
    def description(): # type: ignore
        '''
        Description of the model
        '''
        return "Prepare and postprocess the calcualtion of band structure"
    
    @staticmethod
    def prepare_args(parser):
        '''
        Add arguments for the prepare subcommand
        The arguments can not be command, model, modelcommand '''
        
        parser.description = "Prepare the band structure calculation. Will generate the scf/nscf/kpath input files."
        parser.add_argument('-j', '--job',default=["."], action="extend",nargs="*" ,help='the path of abacus inputs, default is current folder')
        parser.add_argument("-c", "--rundftcommand", type=str, default="OMP_NUM_THREADS=1 mpirun -np 16 abacus | tee out.log",help="the command to execute aabcus, default is 'OMP_NUM_THREADS=1 mpirun -np 16 abacus | tee out.log' ")
        parser.add_argument("-i","--image",default=RECOMMAND_IMAGE,type=str,help="the used image. Default is: registry.dp.tech/dptech/abacus-stable:LTSv3.10", )
        parser.add_argument("--machine", default="c32_m64_cpu", help="the machine to run the abacus. Default is c32_m64_cpu")
        parser.add_argument("-r", "--run", default=0, help="if run the test. Default is 0.", type=int)
        return parser
    
    def run_prepare(self,params):
        '''
        Parse the parameters and run the prepare process.
        Usually, this step will generate the input files for abacustest submit.
        '''
        real_jobs = comm.get_job_list(params.job)
        real_jobs, run_script, extra_files = PrepBand(real_jobs,params.rundftcommand).run()
        if len(real_jobs) == 0:
            print("No valid job is found")
            return
        
        setting = {
            "save_path": "results",
            "bohrium_group_name": "band-strcutre",
            "run_dft": {
                "example": real_jobs,
                "extra_files": extra_files,
                "command": run_script,
                "image": params.image,
                "bohrium": {
                    "scass_type": params.machine,
                    "job_type": "container",
                    "platform": "ali",
                },
            },
        }
        
        comm.dump_setting(setting)
        comm.doc_after_prepare("band", real_jobs, ["setting.json"],has_prepare=False)
        print("After finish the calculation, you can run below command to do the postprocess:")
        print(f"    abacustest model {self.model_name()} post -j {' '.join(real_jobs)} -r result.json\n")
        
        if params.run:
            bash_script = "cal_band_structure.sh"
            with open(bash_script,"w") as f:
                f.write("abacustest submit -p setting.json\n")
            os.system(f"bash {bash_script} &")

    @staticmethod
    def postprocess_args(parser):
        '''
        PostProcess the band structure calculation. Will plot the band structure and calculate the band gap.
        '''
        parser.description = "PostProcess the band structure calculation. Will plot the band structure and calculate the band gap."
        parser.add_argument('-j', '--job',default=[], action="extend",nargs="*" ,help='the path of abacus inputs, default is the current path. There should be some eos folders in each example path.')
        parser.add_argument("--input", default="INPUT.nscf", type=str, help="the input file name, default is INPUT.nscf")
        parser.add_argument("--kpt", default="KPT.nscf", type=str, help="the kpoint file name, default is KPT.nscf")
        parser.add_argument("--range", default=[-5,5], type=float, help="The range of the band structure, default is efermi-5, efermi+5 eV", nargs=2)
        return parser
        
        
    def run_postprocess(self,params):
        '''
        Parse the parameters and run the postprocess process'''
        jobs = comm.get_job_list(params.job)
        PostBand(jobs, input_file=params.input, kpt_file=params.kpt, band_range=params.range).run()
        print("The band structure is plotted in the band.png file")
                
class PrepBand:
    def __init__(self, jobs,run_command=None,job_type="abacus"):
        self.jobs = jobs
        self.job_type = job_type
        self.run_command = self.modify_command(run_command)
        self.scf_input = "INPUT.scf"
        self.scf_kpt = "KPT.scf"
        self.nscf_input = "INPUT.nscf"
        self.nscf_kpt = "KPT.nscf"
        self.run_script = "run.sh"
    
    def run(self):
        if self.job_type == "abacus":
            jobs = self.run_abacus()
        else:
            print(f"Error: the job type {self.job_type} is not supported")
            jobs = []
        
        return jobs, f"bash {self.run_script}", [self.run_script]
                
    def modify_command(self,command):
        # if the command is end with | tee out.log, remove it
        if command is None:
            return "abacus"
        commands = command.split()
        if "|" in commands and "tee" in commands and commands.index("|") == commands.index("tee") - 1:
            return " ".join(commands[:commands.index("|")])  
        else:
            return command        
                   
    def run_abacus(self):
        valid_jobs = []
        for job in self.jobs:
            if not os.path.isdir(job):
                print(f"Warning: {job} is not a valid folder")
                continue
            inputfile = os.path.join(job,"INPUT")
            strufile = os.path.join(job,"STRU")
            if not os.path.isfile(inputfile):
                print(f"Warning: {job} is not a valid folder, can not find {inputfile}")
                continue
            if not os.path.isfile(strufile):
                print(f"Warning: {job} is not a valid folder, can not find {strufile}")
                continue

            try:
                kpt = ReadKpt(job)
            except:
                traceback.print_exc()
                print(f"Warning: {job} is not a valid folder, can not get the K points information from KPT file or INPUT file")
                continue

            input_param = ReadInput(inputfile)
            scf_input = copy.deepcopy(input_param)
            nscf_input = copy.deepcopy(input_param)
            
            stru = AbacusStru.ReadStru(strufile)
            if stru is None:
                print(f"Warning: {job} is not a valid folder, can not get the structure information from STRU file")
                continue

            # generate the scf input
            scf_input["calculation"] = "scf"
            scf_input["out_chg"] = 1
            scf_input["kpoint_file"] = self.scf_kpt
            scf_input.pop("kspacing",None)
            WriteInput(scf_input,os.path.join(job,self.scf_input))
            WriteKpt(kpt[0], os.path.join(job,self.scf_kpt),model=kpt[1])
            
            # generate the nscf input
            nscf_input["calculation"] = "nscf"
            nscf_input["init_chg"] = "file"
            nscf_input["out_band"] = 1
            nscf_input["kpoint_file"] = self.nscf_kpt
            nscf_input.pop("kspacing",None)
            if nscf_input.get("symmetry",None) == 1:
                nscf_input["symmetry"] = 0
            WriteInput(nscf_input,os.path.join(job,self.nscf_input))
            stru.get_kline(kpt_file=os.path.join(job,self.nscf_kpt),point_number=20)
            
            valid_jobs.append(job)
        # gen run script
        if len(valid_jobs) > 0:
            with open(self.run_script,"w") as f:
                f.write(f"cp {self.scf_input} INPUT\n")
                f.write(f"{self.run_command} | tee scf.log\n")
                f.write(f"cp {self.nscf_input} INPUT\n")
                f.write(f"{self.run_command} | tee nscf.log\n")
        return valid_jobs
        

class PostBand:
    def __init__ (self, jobs, input_file=None, kpt_file=None, band_range=None):
        self.jobs = jobs
        self.input_file = input_file 
        self.kpt_file = kpt_file  
        self.band_range = [-5, 5 ] if band_range is None else band_range
                    
    def run(self):
        for job in self.jobs:
            print(job)
            inputfile = self.find_input(job)
            kpt = self.get_kpoints(inputfile, job)
            suffix = "ABACUS"
            if inputfile is not None:
                input_param = ReadInput(inputfile)
                suffix = input_param.get("suffix","ABACUS")
            band_file = os.path.join(job, "OUT."+input_param.get("suffix","ABACUS"), "BANDS_1.dat")
            if not os.path.isfile(band_file):
                print(f"Warning: can not find the band file {band_file}")
                continue
            bands = self.get_band(band_file)
            self.plot_band(bands[1:], kpt, os.path.join(job,"band.png"), 
                           x=bands[0],
                           efermi=self.get_efermi(os.path.join(job,"OUT."+suffix,"running_nscf.log")))
    
    def get_efermi(self, logfile):
        with open(logfile) as f:
            lines = f.readlines()
            
        for line in lines[::-1]:
            if "E_Fermi" in line:
                return float(line.split()[-1])
            elif "read in fermi energy" in line:
                return float(line.split()[-1]) * RY2EV
        return None
                
    @staticmethod
    def get_band(band_file):
        bands = np.loadtxt(band_file)
        return bands[:,1:].T # the first column is the index, the second colume is the coordinate
                 
    def find_input(self, job):
        if self.input_file is not None and os.path.isfile(os.path.join(job,self.input_file)):
            return os.path.join(job,self.input_file)
        else:
            for ifile in ["INPUT.nscf", "INPUT"]:
                if os.path.isfile(os.path.join(job,ifile)):
                    return os.path.join(job,ifile)
        return None
    
    def get_kpoints(self, inputfile, job):
        if self.kpt_file is not None and os.path.isfile(os.path.join(job,self.kpt_file)):
            return ReadKpt(os.path.join(job,self.kpt_file))

        if inputfile is not None:
            input_param = ReadInput(inputfile)
            kpt_file = input_param.get("kpoint_file","KPT")
            if os.path.isfile(os.path.join(job,kpt_file)):
                return ReadKpt(os.path.join(job,kpt_file))
        
        for ifile in ["KPT.nscf", "KPT", "KLINES","LINES"]:
            if os.path.isfile(os.path.join(job,ifile)):
                return ReadKpt(os.path.join(job,ifile))
            
        return None

    @staticmethod
    def split_plotdata(band_in, line_points, x=None):
        # band: Nbands x Nkpoints
        # line_points is a list of the high symmetry k points and contarns the k points and symbols, 
        # which is generated by the ReadKpt.get_kline, like [0.0, 0.0, 0.0, 20, '#G']
        
        band = np.array(copy.deepcopy(band_in))
        nbands, nkpoints = band.shape
        symbols = []
        positions = []
        band_values = []
        
        if x is not None and len(x) != nkpoints:
            print(f"Warning: the length of x is not equal to the length of band data, x={len(x)}, band={len(band[0])}")
            return None
        
        if x is not None:
            x = np.array(x)

        start_position = 0
        start_idx = 0
        for idx in range(len(line_points)-1):
            line = line_points[idx]
            kx, ky, kz, npoint, symbol = line
            if npoint == 1:
                if x is not None:
                    start_idx += 1
                continue 
            
            kx_next, ky_next, kz_next, npoint_next, symbol_next = line_points[idx+1]
            symbols.append([symbol.lstrip("#").strip(),symbol_next.lstrip("#").strip()])
            band_values.append(band[:, start_idx:start_idx+npoint+1])

            if x is None:
                distance = np.sqrt((kx_next-kx)**2 + (ky_next-ky)**2 + (kz_next-kz)**2)  
                positions.append(start_position + np.arange(0, npoint+1) * distance / (npoint))
                start_position += distance
            else:
                positions.append(x[start_idx:start_idx+npoint+1])
                start_idx += npoint

        return positions, symbols, band_values

        '''
        # if one point has only 1 k point, then we should merge it with the next point
        x = sum(npoint) - npoint.count(1)
        if npoint[-1] == 1: x += 1
        
        symbol_index = [0]
        symbols_new = [symbols[0]]
        for i in range(1,len(npoint)):
            if npoint[i-1] == 1:
                continue
            symbol_index.append(symbol_index[-1] + npoint[i-1])
            if npoint[i] == 1 and i != len(npoint) - 1:
                symbols_new.append(symbols[i] + "|" + symbols[i+1])
            else:
                symbols_new.append(symbols[i])
                

        band_idx = [] # we need split the band data into different segments
        x_start = 0  # the start index in x axis
        data_start = 0 # the start index in band data
        points = [npoint[0]]
        for i in range(1, len(npoint)):
            if npoint[i-1] == 1 or i == len(npoint) - 1:
                band_idx.append([x_start, x_start + sum(points), data_start, data_start + sum(points)])
                x_start += sum(points) - 1  # need merge the two points into one
                data_start += sum(points)
                points = [npoint[i]]
            else:
                points.append(npoint[i])
        band_idx[-1][1] += 1
        band_idx[-1][3] += 1
        return band_idx, symbol_index, symbols_new
        '''
    
    def rearrange_label(self, positions, symbols, band_values, new_symbols):
        new_pos = []
        new_bandv = []

        # check if the symbols in new_symbols are in the symbols
        for isymbol in new_symbols:
            if isymbol not in symbols and [isymbol[1], isymbol[0]] not in symbols:
                print(f"Warning: the symbol {isymbol} is not in the symbols")
                return positions, symbols, band_values

        start_pos = 0    
        for i in range(len(new_symbols)):
            s0 = new_symbols[i][0]
            s1 = new_symbols[i][1]
            if [s0,s1] in symbols:
                idx = symbols.index([s0,s1])
                pos = positions[idx]
                bv = band_values[idx]
            elif [s1,s0] in symbols:
                idx = symbols.index([s1,s0])
                pos = positions[idx][::-1] * -1
                bv = band_values[idx][::-1]

            new_pos.append(pos - pos[0] + start_pos)
            new_bandv.append(bv)
            start_pos = new_pos[-1][-1] 
        return new_pos, new_symbols, new_bandv

    
    def plot_band(self, band, kpt, filename, x=None, efermi = None):
        if x is None:
            x = np.arange(len(band[0]))
        if len(x) != len(band[0]):
            print(f"Warning: the length of x is not equal to the length of band data, x={len(x)}, band={len(band[0])}")
            return None
        
        if kpt is None or kpt[1] != "line":
            positions = [np.range(len(band[0]))]
            band_values = [band]
            symbols = None
        else:
            positions, symbols, band_values = self.split_plotdata(band, kpt[0], x)
            # ABACUS has a bug that if the symbols are not continuous, the positions will has a gap
            # so we need to rearrange the symbols and positions
            positions, symbols, band_values = self.rearrange_label(positions, symbols, band_values, symbols)
            #print(symbols)   
        import matplotlib.pyplot as plt
        fontsize = 12
        fig, ax = plt.subplots(1,2,figsize=(8,4))
        for i, (pos, iband) in enumerate(zip(positions, band_values)):
            for iiband in iband:
                ax[0].plot(pos, iiband, "-", color="red",linewidth=0.6)
                ax[1].plot(pos, iiband, "-", color="red",linewidth=0.6)
        
        if symbols is not None:
            symbol_pos = [positions[0][0]]
            symbol_val = [symbols[0][0]]
            for i in range(1,len(positions)):
                symbol_pos.append(positions[i][0])
                if symbols[i-1][-1] == symbols[i][0]:
                    symbol_val.append(symbols[i][0])
                else:
                    symbol_val.append(symbols[i-1][-1] + "|" + symbols[i][0])
                ax[0].axvline(x=positions[i][0], color="gray", linestyle="--", linewidth=0.4)
                ax[1].axvline(x=positions[i][0], color="gray", linestyle="--", linewidth=0.4)
            symbol_pos.append(positions[-1][-1])
            symbol_val.append(symbols[-1][-1])
            print(symbol_pos)
            print(symbol_val)
            ax[0].set_xticks(symbol_pos)
            ax[0].set_xticklabels(symbol_val, fontsize=fontsize)
            ax[1].set_xticks(symbol_pos)
            ax[1].set_xticklabels(symbol_val, fontsize=fontsize)
            
        ax[0].set_xlim(0, positions[-1][-1])
        ax[1].set_xlim(0, positions[-1][-1])  

        ax[0].set_xlabel("K points", fontsize=fontsize)
        ax[1].set_xlabel("K points", fontsize=fontsize)
        ax[0].set_ylabel("Energy (eV)", fontsize=fontsize)
        ax[1].set_ylabel("Energy (eV)", fontsize=fontsize)

        if efermi is not None:
            ax[0].axhline(y=efermi, color="red", linestyle="--", linewidth=0.8)
            ax[1].axhline(y=efermi, color="red", linestyle="--", linewidth=0.8)
            bg = cal_band_gap([np.array(band).T], efermi)
            #ax[0].text(band_idx[-1][1], efermi+0.3, f"E_f={efermi:.2f}eV (BG={bg:.2f}eV)", color="red", fontsize=fontsize-2)
            #ax[1].text(band_idx[-1][1], efermi+0.3, f"E_f={efermi:.2f}eV (BG={bg:.2f}eV)", color="red", fontsize=fontsize-2)
            ax[1].set_ylim(efermi+self.band_range[0],efermi+self.band_range[1])
            title = f"Band Structure (E_fermi={efermi:.2f}eV, BandGap={bg:.2f}eV)"
            print(f"E_fermi={efermi:.2f}eV, BandGap={bg:.2f}eV")
        else:  
            ax[1].set_ylim(self.band_range[0],self.band_range[1])
            title = "Band Structure"
        #ax[0].set_title(title, fontsize=fontsize)
        
        # set a total title
        plt.suptitle(title, fontsize=fontsize)
        
        plt.tight_layout()
        plt.savefig(filename,dpi=400)
        plt.close()
                
        
        
                        
        
        
        

    


import os,glob,json
import copy
import numpy as np
from ..model import Model
from . import comm
from abacustest.lib_prepare.abacus import AbacusStru,ReadInput,WriteInput,WriteKpt
from abacustest.lib_prepare.comm import kspacing2kpt
from abacustest.lib_collectdata.collectdata import RESULT
from abacustest.constant import RECOMMAND_IMAGE

class Eos(Model):
    '''
    Prepare the input files for the EOS calculation
    '''
    @staticmethod
    def model_name():
        '''
        Name of the model, which will be used as the subcommand
        '''
        return "eos"
    
    @staticmethod
    def description():
        '''
        Description of the model
        '''
        return "Prepare and postprocess the EOS calculation"
    
    @staticmethod
    def add_args(parser):
        '''
        Add arguments for the model
        The arguments can not be command, model, modelcommand 
        '''
        pass
    
    def run(self,params):
        '''
        Parse the parameters and run the model
        '''
        pass
    
    @staticmethod
    def prepare_args(parser):
        parser.description = "Prepare the input files for the EOS calculation. Will generate some eos folders in each example path."
        parser.add_argument('-s', '--start', type=float, default=0.9,help="the maximum scaling down of volume, should be less than 1, default is 0.9")
        parser.add_argument('-e', '--end', type=float,  default=1.1,help='the maximum scaling up of volume, should be larger than 1, default is 1.1')
        parser.add_argument('-d', '--step', type=float,  default=0.025,help='the step size of volume scaling, default is 0.025')
        parser.add_argument('-j', '--job',default=["."], action="extend",nargs="*" ,help='the path of abacus inputs, default is the current path.')
        parser.add_argument('--kspacing', default=0, type=int, help="if kspacing is defined in INPUT, then use the this kspacing in all EOS calculation. If set to 0, then will generate the KPT file by kspacing and use this KPT for all EOS calculation. 0: not use; 1: use. default 0")
        parser.add_argument('--relax',default=-1, type=int,help='the calculation type. -1:use the setting in INPUT; 0: scf; 1: relax; 2:cell-relax with fixed volume; default -1')
        parser.add_argument('--clean',default=1, type=int,help='if clean the eos folder in each example path, 0: not clean; 1: clean. default 1')
        #parser.add_argument("-r", "--run", default=0, help="if run the test. Default is 0.", type=int)
    
    def run_prepare(self,params):
        '''
        Parse the parameters and run the prepare process.
        Usually, this step will generate the input files for abacustest submit.
        '''
        jobs = params.job if len(params.job) == 1 else params.job[1:]
        real_jobs = comm.get_job_list(jobs)
        peos = PrepareEos(real_jobs,params.start,params.end,params.step,params.relax,params.clean,params.kspacing)
        subfolders = peos.run()
        
        setting = {
            "save_path": "results",
            "bohrium_group_name": "EOS",
            "run_dft": {
                "example": subfolders,
                "command": "OMP_NUM_THREADS=1 mpirun -np 16 abacus | tee out.log",
                "image": RECOMMAND_IMAGE,
                "bohrium": {
                    "scass_type": "c32_m64_cpu",
                    "job_type": "container",
                    "platform": "ali",
                },
            },
        }
        comm.dump_setting(setting)
        comm.doc_after_prepare("EOS", subfolders, ["setting.json"],has_prepare=False)
    
    @staticmethod
    def postprocess_args(parser):
        '''
        Catch some key values for several EOS jobs, and then do below things:
        1. plot the EOS curve: volume vs energy_per_atom
        2. fit the EOS curve: .
        3. calculate the Delta value between two EOS curves: .
        '''
        parser.description = "Postprocess the EOS calculation. Will plot the EOS curve and fit the EOS curve."
        parser.add_argument('-j', '--job',default=[], action="extend",nargs="*" ,help='the path of abacus inputs, default is the current path. There should be some eos folders in each example path.')
        parser.add_argument("-s", '--save', default="result.json", type=str, help="the file to save the EOS results. Default is result.json")
        parser.add_argument('-r', '--result',default=None, type=str,help='the result file that contains the volume and energy_per_atom of each example. If has defined job, then will deal with both job and result file.')
        parser.add_argument('--ref', default=[], action="extend",nargs="*" , help='the reference result file. Will calculate the Delta value between results and reference. The format should be like result.json')
        parser.add_argument('--sumfile', default=None,const="alleos.png" , type=str, nargs='?',help='if plot all the EOS curves in one figure, then save the figure to this file. Default file name is alleos.png')

    def run_postprocess(self,params):
        '''
        Parse the parameters and run the postprocess process'''
        jobs = params.job 
        result = params.result
        ref = params.ref
        savef = params.save
        sumfile = params.sumfile
        
        if not jobs and not result:
            print("No job or metric is specified.")
            return
        
        PostEos(jobs,result,savef,ref,sumfile).run()

    
class PrepareEos:
    '''
    A class to prepare the input files for the EOS calculation.
    '''
    
    def __init__(self,jobs,start,end,step,relax_type,clean,use_kspacing):
        self.jobs = jobs
        self.start = start
        self.end = end
        self.step = step # the step size of volume scaling, default is 0.025
        self.relax_type = relax_type  # -1:use the setting in INPUT; 0: scf; 1: relax; 2:cell-relax with fixed volume; default -1
        self.clean = clean # if clean the eos folder in each example path, 0: not clean; 1: clean. default 1
        self.use_kspacing = use_kspacing # if kspacing is defined in INPUT, then use the this kspacing in all EOS calculation. If set to 0, then will generate the KPT file by kspacing and use this KPT for all EOS calculation. 0: not use; 1: use. default 0
        self.eos_path = "eos"
        pass
    
    def run(self):
        print("prepare EOS inputs")
        subfolders = []
        for job in self.jobs:
            if self.clean:
                comm.clean_files(job,folder_list=[self.eos_path+"*"])
            subfolders += PrepareEos.prepare_onejob(job,job,self.start,self.end,self.step,self.relax_type,self.use_kspacing,self.eos_path)
        return subfolders

    @staticmethod
    def prepare_onejob(ijob,t_path,start,end,step,relax_type,use_kspacing=False,eos_path="eos"):
        # read the input/STRU/kpt/pp/orb files
        abacus_input = comm.get_abacus_inputfiles(ijob)
        input_param = abacus_input["input"]
        stru = abacus_input["stru"]
        otherfiles = abacus_input["extra_files"]
        kpt = abacus_input["kpt"]
        if abacus_input["pp"]:
            otherfiles += abacus_input["pp"]
        if abacus_input["orb"]:
            otherfiles += abacus_input["orb"]
        
        if not input_param or not stru:
            print(f"ERROR: read input/STRU failed in {ijob}")
            return []

        # modify calculation type
        if relax_type == 0:
            input_param["calculation"] = "scf"
        elif relax_type == 1:
            input_param["calculation"] = "relax"
        elif relax_type == 2:
            input_param["calculation"] = "cell-relax"
            input_param["fixed_axes"] = "volume" 
        
        # set the KPT
        # if kspacing is defined in INPUT, then 
        if "kspacing" in input_param and not use_kspacing:
            kpt = kspacing2kpt(input_param["kspacing"],stru.get_cell(bohr=True))  # three integers
            input_param.pop("kpt_file","")  # remove the kpt_file defined in INPUT

        # write the input file
        subfolders = []
        # write eos0
        new_stru = copy.deepcopy(stru)
        cell = stru.get_cell(bohr=True)
        new_stru.set_cell(cell,bohr=True,change_coord=True)
        spath = os.path.join(t_path,eos_path+"0")
        PrepareEos.write_onejob(new_stru, input_param, kpt, otherfiles, spath)
        subfolders.append(spath)

        # write scaled up/down stru
        number = 1
        while True:
            lcd = 1 - number*step
            lcu = 1 + number*step
            if lcd < start and lcu > end:
                break
            if lcd >= start:
                new_stru = copy.deepcopy(stru)
                new_cell = np.array(cell) * lcd**(1/3)
                new_stru.set_cell(new_cell,bohr=True,change_coord=True)
                spath = os.path.join(t_path,f"{eos_path}{-number}")
                PrepareEos.write_onejob(new_stru, input_param, kpt, otherfiles, spath)
                subfolders.append(spath)
            if lcu <= end:
                new_stru = copy.deepcopy(stru)
                new_cell = np.array(cell) * lcu**(1/3)
                new_stru.set_cell(new_cell,bohr=True,change_coord=True)
                spath = os.path.join(t_path,f"{eos_path}{number}")
                PrepareEos.write_onejob(new_stru, input_param, kpt, otherfiles, spath)
                subfolders.append(spath)
            number += 1
        return subfolders
    
    @staticmethod
    def write_onejob(new_stru, input_param, kpt, extra_files, t_path):
        # wrtie the input file to the t_path
        os.makedirs(t_path,exist_ok=True)
        new_stru.write(os.path.join(t_path,"STRU"))
        WriteInput(input_param,os.path.join(t_path,"INPUT"))
        if isinstance(kpt,list):
            WriteKpt(kpt,os.path.join(t_path,"KPT"))
        elif isinstance(kpt,str) and os.path.isfile(kpt):
            os.symlink(kpt,os.path.join(t_path,os.path.basename(kpt)))
        for ifile in extra_files:
            if os.path.isfile(ifile):
                os.symlink(ifile,os.path.join(t_path,os.path.basename(ifile)))


class PostEos:
    '''
    A class to postprocess the EOS calculation
    '''
    
    def __init__(self,jobs,result,savef,refs,plot_summary_file=None):
        self.jobs = jobs
        self.result = result
        self.savef = savef # the file to save the EOS results which only volume/energy_per_atom
        self.refs = refs
        self.eos_path = "eos"
        self.metrics_file = "metrics.json" # the file to save the metrics which contains more details
        self.plot_summary_file = plot_summary_file # if plot the summary file
    
    def run(self):
        allresults,allmetrics = self.collectdata_result()
        json.dump(allresults,open(self.savef,"w"),indent=4)
        json.dump(allmetrics,open(self.metrics_file,"w"),indent=4)
        result_ref = self.collect_refs()
        PostEos.plot_eos(allresults,result_ref,self.plot_summary_file)
        
    def collectdata_result(self):
        allresults = {}  # key is {jobname}, value is volume and energy_per_atom
        allmetrics = {}  # key is {jobname/eos*}, value is volume, energy_per_atom, converge, normal_end, total_time, scf_steps, version...
        if self.result:
            if os.path.isfile(self.result):
                allresults = json.load(open(self.result))
        
        if self.jobs:
            for ijob in self.jobs:
                vs = []
                es = []
                alleos = glob.glob(f"{ijob}/{self.eos_path}*")
                for ieos in alleos:
                    if os.path.isdir(ieos):
                        print("collecting",ieos)
                        r = RESULT(path=ieos,fmt="abacus")
                        if None not in [r["volume"], r["energy_per_atom"]]:
                            vs.append(r["volume"])
                            es.append(r["energy_per_atom"])
                        allmetrics[ieos] = {
                            "volume": r["volume"],
                            "energy_per_atom": r["energy_per_atom"],
                            "natom": r["natom"],
                            "converge": r["converge"],
                            "normal_end": r["normal_end"],
                            "relax_converge": r["relax_converge"],
                            "relax_steps": r["relax_steps"],
                            "total_time": r["total_time"],
                            "scf_steps": r["scf_steps"],
                            "version": r["version"]
                        }
                if vs:
                    allresults[ijob.rstrip("/")] = {"volume":vs,"energy_per_atom":es}
        return allresults,allmetrics
    
    def collect_refs(self):
        result_ref = {}
        if self.refs:
            for iref in self.refs:
                if os.path.isfile(iref):
                    result_ref.update(json.load(open(iref)))
        return result_ref

    @staticmethod
    def plot_eos(results, ref_results={}, one_plot_file=None):
        '''
        Plot the EOS curve,
        
        Args:
            results (dict): The results of the EOS calculation. The key is the jobname, the value is a dict of volume and energy_per_atom.
            ref_results (dict): The reference results of the EOS calculation. The key is the jobname, the value is the volume and energy_per_atom.
            one_plot_file (str): The file to save the EOS curve. If not defined, then will show the EOS curve.
        
        If ref_results is defined, then will also plot the reference EOS curve, and calculate the Delta value between the results and reference.
        If one_plot_file is defined, then will also plot all the EOS curves in one figure.
        '''
        from . import comm_eos
        import matplotlib.pyplot as plt
        
        examples = []
        for k,v in results.items():
            if v and "volume" in v and "energy_per_atom" in v and len(v["volume"]) == len(v["energy_per_atom"]):
                examples.append(k)
            else:
                print(f"Example {k} has no valid results. Need keys: volume and energy_per_atom.")
                
        n = len(examples)
        if n == 0:
            print("No example to plot.")
            return
        
        # save plot data to a csv file
        plot_data = ""
        for ik, k in enumerate(examples):
            fig, ax = plt.subplots(1,1,figsize=(8,5))
            plot_r = comm_eos.plot_eos_one(ax,results[k], ref_results.get(k,None), label_size=16, legend_size=14,x_label="Volume ($A^3$)",y_label="Energy ($eV/atom$)")
            title = "EOS of " + k
            if plot_r and plot_r.get("delta",None):
                delta = plot_r["delta"]
                title += f" (Delta={delta:.2e})"
            ax.set_title(title,fontsize=16)
            
            filename = k.strip("/").replace("/","_") + ".png"
            plt.tight_layout()
            plt.savefig(filename)
            plt.close()
                      
            plot_data += f"Example {k}:\n"
            plot_data += "x," + ",".join([f"{x:.2f}" for x in results[k]["volume"]]) + "\n"
            plot_data += "y," + ",".join([f"{x:.6f}" for x in results[k]["energy_per_atom"]]) + "\n"
            if ref_results.get(k,None):
                plot_data += "x_ref," + ",".join([f"{x:.2f}" for x in ref_results[k]["volume"]]) + "\n"
                plot_data += "y_ref," + ",".join([f"{x:.6f}" for x in ref_results[k]["energy_per_atom"]]) + "\n"
            if plot_r:
                if plot_r["fit"]:
                    v0, e0, fitx, fity, b0, bp, res = plot_r["fit"]
                    plot_data += "fitx," + ",".join([f"{x:.2f}" for x in fitx]) + "\n"
                    plot_data += "fity," + ",".join([f"{x:.6f}" for x in fity]) + "\n"
                    plot_data += f"v0,{v0:.2f}\n"
                    plot_data += f"e0,{e0:.6f}\n"
                    plot_data += f"b0,{b0:.6f}\n"
                    plot_data += f"bp,{bp:.6f}\n"
                    plot_data += f"res,{res:.6e}\n"
                if plot_r["fit_ref"]:
                    v0_ref, e0_ref, fitx_ref, fity_ref, b0_ref, bp_ref, res_ref = plot_r["fit_ref"]
                    plot_data += "fitx_ref," + ",".join([f"{x:.2f}" for x in fitx_ref]) + "\n"
                    plot_data += "fity_ref," + ",".join([f"{x:.6f}" for x in fity_ref]) + "\n"
                    plot_data += f"v0_ref,{v0_ref:.2f}\n"
                    plot_data += f"e0_ref,{e0_ref:.6f}\n"
                    plot_data += f"b0_ref,{b0_ref:.6f}\n"
                    plot_data += f"bp_ref,{bp_ref:.6f}\n"
                    plot_data += f"res_ref,{res_ref:.6e}\n"
                if plot_r.get("delta",None):
                    plot_data += f"delta,{plot_r['delta']:.6e}\n"
            plot_data += "\n"
        
        with open("plot_data.csv","w") as f:
            f.write(plot_data)  
        
        def flat_list(l):
            a = []
            if not isinstance(l,list):
                return [l]
            for i in l:
                if isinstance(i,list):
                    a += flat_list(i)
                else:
                    a.append(i)
            return a
        
        if one_plot_file:
            ncol = int(n ** 0.5)   
            nrow = int(n / ncol) 
            while ncol * nrow < n:
                ncol += 1
            figs, axs = plt.subplots(nrow, ncol, figsize=(ncol*8, nrow*5))
            axs_flat = flat_list(axs)
            
            for ik, k in enumerate(examples):
                ax = axs_flat[ik]
                plot_r = comm_eos.plot_eos_one(ax,results[k], ref_results.get(k,None), label_size=16, legend_size=14,x_label="Volume ($A^3$)",y_label="Energy ($eV/atom$)")
                title = "EOS of " + k
                if plot_r:
                    delta = plot_r.get("delta",None)
                    if delta != None:
                        title += f" (Delta={delta:.2e})"
                ax.set_title(title)
            plt.tight_layout()
            plt.savefig(one_plot_file)
            plt.close()

        
            
            
            
    
    
        

    
        
        
        
        
        
        


from ..model import Model
from . import comm
import argparse,json, os
from abacustest.lib_prepare.abacus import WriteKpt, WriteInput, gen_stru, ReadInput
from pathlib import Path
from abacustest.lib_model.model_012_band import PrepBand
from abacustest.constant import RECOMMAND_IMAGE

from typing import List, Dict, Any
from pathlib import Path


JOB_TYPES = {"scf": {"calculation": "scf", "symmetry": 1, "ecutwfc": 80, "scf_thr": 1e-8, "scf_nmax": 100,
                    "smearing_method": "gauss", "smearing_sigma": 0.015, "mixing_type": "broyden",
                    "mixing_beta": 0.7,  "basis_type": "pw  # or lcao", "ks_solver": "dav_subspace  # or genelpa for lcao basis",
                    "#cal_force": 1, "#cal_stress": 1,
                    "kspacing": "0.1 # unit in 1/bohr"}, 
             "relax": {"calculation": "relax", "symmetry": 1, "ecutwfc": 80, "scf_thr": 1e-8, "scf_nmax": 100,
                       "smearing_method": "gauss", "smearing_sigma": 0.015, "mixing_type": "broyden",
                       "mixing_beta": 0.7, "basis_type": "pw  # or lcao", "ks_solver": "dav_subspace  # or genelpa for lcao basis",
                       "cal_force": 1, "#cal_stress": 1,"kspacing": "0.1 # unit in 1/bohr",
                       "relax_method": "cg # or bfgs bfgs_trad cg_bfgs sd fire",
                       "relax_nmax": 60, "force_thr_ev": "0.01  # unit in eV/A", "#stress_thr": "0.5 # unit in kbar",
                       }, 
             "cell-relax":{"calculation": "cell-relax", "symmetry": 1, "ecutwfc": 80, "scf_thr": 1e-8, "scf_nmax": 100,
                       "smearing_method": "gauss", "smearing_sigma": 0.015, "mixing_type": "broyden",
                       "mixing_beta": 0.7, "basis_type": "pw  # or lcao", "ks_solver": "dav_subspace  # or genelpa for lcao basis",
                       "cal_force": 1, "cal_stress": 1, "kspacing": "0.1 # unit in 1/bohr",
                       "relax_method": "cg # or bfgs, bfgs_trad, cg_bfgs, sd, fire",
                       "relax_nmax": 60, "force_thr_ev": "0.01  # unit in eV/A", "stress_thr": "0.5 # unit in kbar",
                       "fixed_axes": "None # or volume, shape, a, b, c, ab, ac, bc; only valid for cell-relax calculation to fix some axes",
                       }, 
             "md":{"calculation": "md", "symmetry": 1, "ecutwfc": 80, "scf_thr": 1e-8, "scf_nmax": 100,
                       "smearing_method": "gauss", "smearing_sigma": 0.015, "mixing_type": "broyden",
                       "mixing_beta": 0.7, "basis_type": "pw  # or lcao", "ks_solver": "dav_subspace  # or genelpa for lcao basis",
                       "#cal_force": 1, "#cal_stress": 1,
                       "kspacing": "0.1 # unit in 1/bohr", 
                       "md_type": "nvt  # or npt, nve, langevin, fire, msst",
                       "md_nstep": "10  # number of steps",
                       "md_dt": "1.0  # unit in fs", 
                       "md_tfirst": "100  # unit in K",
                       "md_tlast": "100  # unit in K",}, 
             "band":{"calculation": "scf", "symmetry": 1, "ecutwfc": 80, "scf_thr": 1e-8, "scf_nmax": 100,
                    "smearing_method": "gauss", "smearing_sigma": 0.015, "mixing_type": "broyden",
                    "mixing_beta": 0.7,  "basis_type": "pw  # or lcao", "ks_solver": "dav_subspace  # or genelpa for lcao basis",
                    "#cal_force": 1, "#cal_stress": 1,
                    "kspacing": "0.1 # unit in 1/bohr"}
        }

LCAO_PARAM = {
    "basis_type": "lcao",
    "ks_solver": "genelpa",
    "#gamma_only": 0,
    "ecutwfc": 100,
    "scf_thr": 1e-7,
}


class InputsModel(Model):
    @staticmethod
    def model_name(): # type: ignore
        '''
        Name of the model, which will be used as the subcommand
        '''
        return "inputs"
    
    @staticmethod
    def description(): # type: ignore
        '''
        Description of the model
        '''
        return "Prepare the ABACUS inputs of specified model"
    
    @staticmethod
    def add_args(parser):
        '''
        Add arguments for the prepare subcommand
        The arguments can not be command, model, modelcommand '''
        
        parser.description = "Prepare the ABACUS inputs file."
        parser.add_argument('-f', '--file',default=[], action="extend",nargs="*" ,help='the structure files')
        parser.add_argument("--ftype",default="cif",type=str,help="the structure type, should be cif or dpdata supportted type like: poscar, abacus/stru, ..", )
        parser.add_argument("--jtype", default="scf", type=str, help=f"the job type, should be one of {list(JOB_TYPES.keys())}")
        parser.add_argument("--pp",default=None,type=str,help="the path of pseudopotential library, or read from enviroment variable ABACUS_PP_PATH")
        parser.add_argument("--orb",default=None,type=str,help="the path of orbital library, or read from enviroment variable ABACUS_ORB_PATH")
        parser.add_argument("--input",default=None,type=str,help="the template of input file, if not specified, the default input will be generated")
        parser.add_argument("--kpt", default=None, type=int, nargs="*", help="the kpoint setting, should be one or three integers")
        parser.add_argument("--lcao", action="store_true", help="whether to use lcao basis, default is pw basis")
        return parser
    
    def run(self,params):
        if params.kpt is not None and len(params.kpt) not in [1, 3]:
            raise ValueError("The kpoint setting should be one or three integers.")
        pinput = PrepInput(
            files=params.file,
            filetype=params.ftype,
            jobtype=params.jtype,
            pp_path=params.pp,
            orb_path=params.orb,
            input_file=params.input,
            kpt=params.kpt,
            lcao=params.lcao,
        )
        pinput.run()
        return 0

class PrepInput:
    def __init__(self, files, filetype, jobtype, pp_path=None, orb_path=None, input_file=None, kpt=None,
                 abacus_command="OMP_NUM_THREADS=1 mpirun -np 16 abacus", machine="c32_m64_cpu", 
                 image=RECOMMAND_IMAGE,
                 lcao=False):
        if jobtype not in JOB_TYPES:
            raise ValueError(f"Unsupported job type: {jobtype}.\nSupported job types are {list(JOB_TYPES.keys())}.")
        
        if not isinstance(files, list):
            self.files = [files]
        else:
            self.files = files
            
        self.filetype = filetype
        self.jobtype = jobtype
        self.pp_path = pp_path
        self.orb_path = orb_path
        self.input_file = input_file
        self.kpt = kpt
        self.abacus_command = abacus_command
        self.machine = machine
        self.image = image
        self.lcao = lcao
        
        if self.pp_path is None and os.environ.get("ABACUS_PP_PATH") is not None:
            self.pp_path = os.environ["ABACUS_PP_PATH"]
        if self.orb_path is None and os.environ.get("ABACUS_ORB_PATH") is not None:
            self.orb_path = os.environ["ABACUS_ORB_PATH"]
        
        print("Structure files:", self.files)
        print("Structure type:", self.filetype)
        print("Job type:", self.jobtype)
        print("Pseudopotential path:", self.pp_path)
        print("Orbital path:", self.orb_path)
        print("Input file:", self.input_file)
        print("Kpoint:", self.kpt)
        print("")
    
    def run(self):
        recommanded_param = JOB_TYPES[self.jobtype]
        auto_ecutwfc = True
        if self.lcao:
            recommanded_param.update(LCAO_PARAM)
            auto_ecutwfc = False
        
        if self.input_file is None:
            print("Automatically generate the input file.")
            input_param = recommanded_param
        else:
            input_param = ReadInput(self.input_file)
            if "calculation" in input_param and input_param.get("calculation") != recommanded_param["calculation"]:
                print(f"Warning: the calculation type in the input file is {input_param['calculation']}, but the job type is {self.jobtype}.")
                print(f"         Automatically set the calculation type to {recommanded_param['calculation']}.")
                input_param["calculation"] = recommanded_param["calculation"]
            if "ecutwfc" in input_param:
                auto_ecutwfc = False
            
            # update other required parameters
            for key in recommanded_param:
                if key in input_param or (key.startswith("#") and key[1:] in input_param):
                    continue
                input_param[key] = recommanded_param[key]
            
        jobs = self.gen_abacus_inputs(self.files, self.filetype, 
                                      self.pp_path, self.orb_path, 
                                      input_param, self.kpt,
                                      auto_ecutwfc=auto_ecutwfc)
        job_path = list(jobs.keys())
        if self.jobtype == "band":
            prep_band = PrepBand(job_path, run_command=self.abacus_command)
            _, run_script, extra_files = prep_band.run()
        else:
            run_script_file = "run.sh"
            Path(run_script_file).write_text(self.abacus_command)
            run_script = f"bash {run_script_file}"
            extra_files = [run_script_file]
            

        # generate the parameter setting for abacustest submit
        setting = {
            "save_path": "results",
            "run_dft": {
                "example": job_path,
                "extra_files": extra_files,
                "command": run_script,
                "image": self.image,
                "bohrium": {
                    "scass_type": self.machine,
                    "job_type": "container",
                    "platform": "ali"
                }
            }
        }
        
        setting_file = "setting.json"
        json.dump(setting, open(setting_file, "w"), indent=4)
        print("\nThe inputs are generated in", ", ".join(job_path))
        print(f"Please modify the ABACUS running command in script {extra_files[0]} if needed.")
        print(f"You can modify '{setting_file}', and execute the command 'abacustest submit -p setting.json' to run the abacustest to submit all jobs to bohrium.")
        print(f"Or you can 'cd' to each job path and execute '{extra_files[0]}' to run the job.")
        
        return setting, job_path
    
    @staticmethod
    def gen_abacus_inputs(stru_files, stru_type, pp_path, orb_path, input_param=None, kpoint=None, auto_ecutwfc=True):
        """
        Generate the abacus input files.

        Parameters
        ----------
        stru_files : list
            The structure files, should be a list of cif or dpdata supported data
        stru_type : str
            The structure type, cif or dpdata supported format
        pp_path : str
            The pseudopotential path
        orb_path : str
            The orbital path
        input_param : dict
            The specified input parameter, which will used in INPUT
        kpoint : list
            The kpoint setting, should be a list of three int, will generate a KPT file
        auto_ecutwfc : bool
            Whether to automatically find the ecutwfc from ecutwfc.json file, default is True

        Returns
        -------
        job_path : dict
            The job path, which is a dict, key is the job path, value is {"element": element, "pp": pp, "orb": orb}
        """
        job_path = gen_stru(stru_files, stru_type, pp_path, orb_path, tpath=".")
        # job_path is dict: key is the job path, value is {"element": element, "pp": pp, "orb": orb}
        # the pp and orb file has been linked to the job path

        if job_path is None or len(job_path) == 0:
            raise ValueError("No valid structure file found.")

        # write INPUT file and KPT file

        default_input = {
            "ecutwfc": 100,
            "calculation": "scf",
            "basis_type": "pw"
        }
        if input_param is not None:
            default_input.update(input_param)
        # will find the recommand ecutwfc from ecutwfc.json file
        if pp_path is not None and os.path.isfile(os.path.join(pp_path, "ecutwfc.json")):
            recommand_ecutwfc = json.load(open(os.path.join(pp_path, "ecutwfc.json"), "r"))
        else:
            recommand_ecutwfc = None

        if auto_ecutwfc and recommand_ecutwfc is not None:
            print("Based on recommended ecutwfc, automatically set ecutwfc for each job.")
            
        for path, job in job_path.items():
            element = job["element"]
            if auto_ecutwfc and recommand_ecutwfc is not None:
                ecutwfcs = []
                for e in element:
                    if e in recommand_ecutwfc:
                        ecutwfcs.append(recommand_ecutwfc[e])
                    else:
                        print(f"Warning: No recommended ecutwfc for element {e}.")
                if len(ecutwfcs) == 0:
                    print(f"Warning: No recommended ecutwfc for elements {element}. Using default ecutwfc {default_input['ecutwfc']}.")
                else:      
                    default_input["ecutwfc"] = max(ecutwfcs)
                    recutwfc = {i: recommand_ecutwfc.get(i,None) for i in element}
                    print(f"{path}: set ecutwfc to {default_input['ecutwfc']}, recommended ecutwfc: {recutwfc}")

            if kpoint is not None:
                default_input.pop("kspacing",None)
                WriteKpt(kpoint, os.path.join(path, "KPT"))
            
            WriteInput(default_input, os.path.join(path, "INPUT"))

        return job_path


import os,sys
from . import abacus as MyAbacus
from abacustest import constant
from . import comm
import copy

def ParamAbacus2Vasp(abacus_input):
    '''
    Generate the vasp input from abacus input.
    
    As the parameters are not totally related, so we just convert the parameters we need.
    - calculation
        - SCF
        - RELAX
        - CELL_RELAX
        - NSCF
    - smearing
    - symmetry
    - kspacing
    
    Below cases will not be considered:
    fixed_axes/
    
    for scf_thr:
    pw: decrease 1e-2
        < 1e-8 -> 1e-6
    lcao: decrease 1e-1
        < 1e-7 -> 1e-6
    '''
    vasp_input = {}
    vasp_input["SYSTEM"] = abacus_input.pop("suffix","VASP")
    vasp_input["LREAL"] = "Auto"
    vasp_input["PREC"] = "Normal"
    vasp_input["ALGO"] = "Normal"
    
    # job type
    calculation = abacus_input.pop("calculation","scf")
    force = abacus_input.pop("force",False)
    stress = abacus_input.pop("stress",False)

    if "ecutwfc" in abacus_input:
        vasp_input["ENCUT"] = abacus_input.pop("ecutwfc") * constant.RY2EV
    
    basis_type = abacus_input.pop("basis_type","pw")
    if basis_type == "pw":
        vasp_input["EDIFF"] = abacus_input.pop("scf_thr",1e-9)/1E-2
    else:
        vasp_input["EDIFF"] = abacus_input.pop("scf_thr",1e-7) / 1E-1
    
    if calculation == "scf":
        vasp_input["IBRION"] = -1
        if stress:
            vasp_input["ISIF"] = 2
        else:
            vasp_input["ISIF"] = 0
    elif calculation in ["relax","cell-relax"]:
        vasp_input["IBRION"] = 2
        if calculation == "relax":
            vasp_input["ISIF"] = 2
        else:
            fixed_axes = abacus_input.pop("fixed_axes","None")
            if fixed_axes == "None":
                vasp_input["ISIF"] = 3
            elif fixed_axes == "volume":
                vasp_input["ISIF"] = 4
            elif fixed_axes == "shape":
                vasp_input["ISIF"] = 7
            else:
                vasp_input["ISIF"] = 3  # FOR other cases, just set to 3
    elif calculation == "md":
        vasp_input["IBRION"] = 0
    elif calculation == "nscf":
        vasp_input["IBRION"] = -1
    else:
        raise ValueError(f"calculation {calculation} is not supported.")
    
    if "scf_nmax" in abacus_input:
        vasp_input["NELM"] = abacus_input.pop("scf_nmax",100)
    
    smearing_method = abacus_input.pop("smearing_method",None)    
    if smearing_method != None:
        if smearing_method == "gaussian":
            vasp_input["ISMEAR"] = 0
        elif smearing_method == "mp":
            vasp_input["ISMEAR"] = 1
        elif smearing_method == "mp2":
            vasp_input["ISMEAR"] = 2
        elif smearing_method == "fd":
            vasp_input["ISMEAR"] = -1
        
        vasp_input["SIGMA"] = abacus_input.pop("smearing_sigma",0.015)*constant.RY2EV

    if "symmetry" in abacus_input:
        symmetry = abacus_input.pop("symmetry")
        if symmetry == 0:
            vasp_input["ISYM"] = 0
        elif symmetry == 1:
            vasp_input["ISYM"] = 1
        elif symmetry == -1:
            vasp_input["ISYM"] = -1
        else:
            raise ValueError(f"symmetry {abacus_input['symmetry']} is not supported.")
    
    if calculation in ["relax","cell-relax"]:
        if "force_thr_ev" in abacus_input: 
            vasp_input["EDIFFG"] = abacus_input.pop("force_thr_ev")*-1
        if "force_thr" in abacus_input:
            vasp_input["EDIFFG"] = abacus_input.pop("force_thr")*-1*constant.RY2EV
           
    if "kspacing" in abacus_input:
        kspacing = abacus_input.pop("kspacing")
        if isinstance(kspacing,list):
            kspacing = sum(kspacing)/3
        elif isinstance(kspacing,str):
            kspacing = kspacing.split()
            if len(kspacing) == 1:
                kspacing = float(kspacing[0])
            elif len(kspacing) == 3:
                kspacing = sum([float(i) for i in kspacing])/3
            else:
                raise ValueError(f"kspacing {kspacing} is not supported.")
        vasp_input["KSPACING"] = kspacing / constant.BOHR2A
        
    if calculation in ["relax","cell-relax"] and "relax_nmax" in abacus_input:
        vasp_input["NSW"] = abacus_input.pop("relax_nmax")
    
    if calculation == "md" and "md_nstep" in abacus_input:
        vasp_input["NSW"] = abacus_input.pop("md_nstep")   
    
    if "nspin" in abacus_input:
        vasp_input["ISPIN"] = abacus_input.pop("nspin")
        if vasp_input["ISPIN"] == 4: vasp_input["ISPIN"] = 2
        if vasp_input["ISPIN"] in [2,4]:
            vasp_input["LORBIT"] = 11 # OUTPUT THE MAGNETIC MOMENT of each atom
    
    if "dft_plus_u" in abacus_input:
        if comm.IsTrue(abacus_input.pop("dft_plus_u")):
            vasp_input["LDAU"] = ".TRUE."
            vasp_input["LDAUTYPE"] = 2
            if "orbital_corr" in abacus_input:
                vasp_input["LDAUL"] = abacus_input.pop("orbital_corr")
                if "2" in vasp_input["LDAUL"]:
                    vasp_input["LMAXMIX"] = 4
                if "3" in vasp_input["LDAUL"]:
                    vasp_input["LMAXMIX"] = 6
            if "hubbard_u" in abacus_input:
                vasp_input["LDAUU"] = abacus_input.pop("hubbard_u")
                vasp_input["LDAUJ"] = " ".join(["0" for i in vasp_input["LDAUU"].split()])
        else:
            vasp_input["LDAU"] = ".FALSE."
            abacus_input.pop("orbital_corr",None)
            abacus_input.pop("hubbard_u",None)
    
    if "nupdown" in abacus_input:
        vasp_input["NUPDOWN"] = abacus_input.pop("nupdown")
    
    # lspinorb and nonclinear
    if "lspinorb" in abacus_input:
        if comm.IsTrue(abacus_input.pop("lspinorb")):
            vasp_input["LSORBIT"] = ".TRUE."
            vasp_input["ISYM"] = -1
        else:
            vasp_input["LSORBIT"] = ".FALSE."
    if "noncolin" in abacus_input:
        if comm.IsTrue(abacus_input.pop("noncolin")):
            vasp_input["LNONCOLLINEAR"] = ".TRUE."
        else:
            vasp_input["LNONCOLLINEAR"] = ".FALSE."
            
    # delta spin
    if "sc_mag_switch" in abacus_input:
        if comm.IsTrue(abacus_input.pop("sc_mag_switch")):
            vasp_input["SCTYPE"] = 1
            if "nsc" in abacus_input:
                vasp_input["NSC"] = abacus_input.pop("nsc")
            if "nsc_min" in abacus_input:
                vasp_input["NSCMIN"] = abacus_input.pop("nsc_min")
            if "sc_thr" in abacus_input:
                vasp_input["SCDIFF"] = abacus_input.pop("sc_thr")
            if "alpha_trial" in abacus_input:
                vasp_input["INISC"] = abacus_input.pop("alpha_trial")
            if "sccut" in abacus_input:
                vasp_input["SCCUT"] = abacus_input.pop("sccut")
        else:
            vasp_input["SCTYPE"] = 0
            abacus_input.pop("nsc",None)
            abacus_input.pop("nsc_min",None)
            abacus_input.pop("sc_thr",None)
            abacus_input.pop("alpha_trial",None)
            abacus_input.pop("sccut",None)
    
    if len(abacus_input) > 0:
        print("WARNING: The following parameters are not converted to VASP:")
        for ip in abacus_input.keys():
            if ip in ["kspacing"]: continue
            print("         %s" % ip)
    
    return vasp_input

def GenIncarFromStru(stru,vasp_setting):
    # if nspin != 1, then set the initial magmom
    if vasp_setting.get("ISPIN",1) != 1:
        mag = stru.get_atommag()  # mag is a list of list, eahc list is the magmom of one atom, for noncolinear, it should be [magx,magy,magz]
        labels = stru.get_label(total=True)
        label = []
        for i in labels:
            if i not in label:
                label.append(i)
        magmom = []
        if vasp_setting.get("LNONCOLLINEAR",False):
            # should create the magmom, a lisf of 3*N, N is the number of atoms
            for imag in mag:
                if isinstance(imag,float):
                    magmom += [0,0,imag]
                elif isinstance(imag,list):
                    if len(imag) == 1:
                        magmom += [0,0,imag[0]]
                    elif len(imag) == 3:
                        magmom += imag
                    else:
                        print("ERROR: The magmom should be a list of 1 or 3!!!")
                        sys.exit(1)
                else:
                    print("ERROR: The magmom should be a list of float or list!!!")
                    sys.exit(1)
        else:
            for imag in mag:
                if isinstance(imag,float):
                    magmom.append(imag)
                elif isinstance(imag,list):
                    if len(imag) == 1:
                        magmom.append(imag[0])
                    elif len(imag) == 3:
                        magmom.append((imag[0]**2 + imag[1]**2 + imag[2]**2)**0.5)
                    else:
                        print("ERROR: The magmom should be a list of 1 or 3!!!")
                        sys.exit(1)
                else:
                    print("ERROR: The magmom should be a list of float!!!")
                    sys.exit(1)
        
        vasp_setting["MAGMOM"] = VaspList2String(magmom)
        
        if comm.IsTrue(vasp_setting.get("SCTYPE")):
            # if sc_mag_switch is true, then set the initial magmom
            vasp_setting["M_CONSTR"] = vasp_setting["MAGMOM"]
            constrain = stru.get_constrain()
            lambda_ = stru.get_lambda()
            c = []
            l = []
            if not vasp_setting.get("LNONCOLLINEAR",False):
                for ic in constrain:
                    if isinstance(ic,list):
                        if False in ic:
                            c.append(0)
                    else:
                        c.append(1 if ic else 0)
                for il in lambda_:
                    if isinstance(il,list):
                        l += il
                    else:
                        l.append(il)
            else:
                for ic in constrain:
                    if isinstance(ic,bool):
                        c += [1 if ic else 0]*3
                    else:
                        c += [1 if i else 0 for i in ic]
                for il in lambda_:
                    if isinstance(il,(float,int)):
                        l += [il]*3
                    elif isinstance(il,list):
                        l += il
                    else:
                        print("ERROR: The lambda should be a list of float or list!!!")
                        sys.exit(1)

            vasp_setting["CONSTRL"] = VaspList2String(c)
            vasp_setting["LAMBDA"] = VaspList2String(l)

def VaspList2String(list1):
    # convert the list1 to string
    # need to merge the same value like: [1, 1, 1, 0, 0, 0, 0] to "3*1 4*0"
    #list1 = [str(i) for i in list1]
    if len(list1) == 0:
        return ""
    
    c = ""
    v = list1[0]
    n = 1
    for i in list1[1:]:
        if i == v or (isinstance(i,(float,int)) and isinstance(v,(float,int)) and abs(i-v) < 1E-6):
            n += 1
        else:
            if n == 1:
                c += f"{v} "
            else:
                c += f"{n}*{v} "
            v = i
            n = 1
    if n == 1:
        c += f"{v} "
    else:
        c += f"{n}*{v} "
    return c
    
    

def KptAbacus2Vasp(abacus_kpt,vasp_kpt="KPOINTS"):
    """
    Convert the kpt from abacus to vasp.
    
    Parameters
    ----------
    abacus_kpt : str
        the kpt file of abacus.
    vasp_kpt : str
        the file to save the vasp kpt.
        
    Only support the automatic kpt.
    """
    if not os.path.isfile(abacus_kpt):
        print(f"ERROR: Not find kpt file {abacus_kpt}!!!")
        sys.exit(1)
    with open(abacus_kpt) as f: aba_kpt = f.readlines()
    
    model = aba_kpt[2].strip().lower()
    
    if model in ["gamma","mp"]:
        kpt = [int(i) for i in aba_kpt[3].split()[:3]]
        shift_ = [float(i) for i in aba_kpt[3].split()[3:6]]

        # write the kpt to vasp_kpt
        cc = "K_POINTS\n0\nGamma\n"
        cc += " ".join([str(i) for i in kpt])+"\n"
        cc += " ".join([str(i) for i in shift_])
        if vasp_kpt != None:
            with open(vasp_kpt,"w") as f:
                f.write(cc)
        return cc       
    else:
        print(f"ERROR: Not support kpt model {model}!!!")
        return ""

def gen_potcar(potcar,label,save_path="POTCAR"):
    """
    Generate the potcar from potcar.
    
    Parameters
    ----------
    potcar : str|dict
        the path of potcar.
        If is a str, it should be the path of PBE-PAW POTCARS, and will use the potcar recommended by vaspwiki for each element.
            https://www.vasp.at/wiki/index.php/Available_PAW_potentials 
        If is a dict, it should be the dict of potcar, and the key is the label of atoms, and the value is the path of potcar.
    label : list
        the label of atoms.
    save_path : str
        the path to save the potcar.
    """
    if potcar == None:
        print("ERROR: Not find potcar!!!")
        return None
    print("potcar/label:",potcar,label)
    all_pots = []
    if isinstance(potcar,str):
        if not os.path.isdir(potcar):
            print(f"ERROR: Not find potcar folder {potcar}!!!")
            return None
        from .vasp import RECOMMEND_PAW
        for i in label:
            if i not in RECOMMEND_PAW:
                print(f"ERROR: Not find potcar for {i}!!!")
                return None
            ipot = os.path.join(potcar,RECOMMEND_PAW[i],"POTCAR")
            if not os.path.isfile(ipot):
                print(f"ERROR: Not find potcar for {i}: {ipot}!!!")
                return None
            all_pots.append(ipot)
    elif isinstance(potcar,dict):
        for i in label:
            if i not in potcar:
                print(f"ERROR: Not find potcar for {i}!!!")
                return None
            ipot = potcar[i]
            if not os.path.isfile(ipot):
                print(f"ERROR: Not find potcar for {i}: {ipot}!!!")
                return None
            all_pots.append(ipot)
    else:
        print("ERROR: potcar must be str or dict!!!")
        return None

    emax = []
    potcar_cc = ""
    for ipot in all_pots:
        with open(ipot) as g:
            lines = g.readlines()
            potcar_cc += "".join(lines)
            for iline in lines:
                if iline.startswith("   ENMAX  ="):
                    emax.append(float(iline.split(";")[0].split("=")[1]))
                    #   ENMAX  =  250.000; ENMIN  =  200.000 eV
                    break
    if save_path != None:
        with open(save_path,"w") as f:
            f.write(potcar_cc)
    emax = max(emax)
    return emax    
    
def Abacus2Vasp(abacus_path:str, save_path:str=None, potcar=None, vasp_setting={}):
    """
    Convert the abacus input to vasp input.
    
    Need convert four files:
    INPUT -> INCAR
    STRU -> POSCAR
    KPT -> KPOINTS
    POTCAR          # the prepare of this file based on potcar
    
    Parameters
    ----------
    abacus_path : str
        the path of abacus input.
    save_path : str
        the path to save vasp input.
    potcar : str|dict
        the path of potcar.
    emax_coef : float
        the coefficient to calculate the ENCUT. 
            ENCUT = emax * emax_coef (default is 1.5)
            emax is the max ENMAX in POTCAR.
    """
    vasp_setting = copy.deepcopy(vasp_setting)
    abacus_input = MyAbacus.ReadInput(os.path.join(abacus_path,"INPUT"))
    stru = MyAbacus.AbacusStru.ReadStru(os.path.join(abacus_path,abacus_input.get("stru_file","STRU")))
    
    # 1. write KPT
    if abacus_input.get("basis_type","lcao") and abacus_input.get("gamma_only",False):
        kpoints = "K_POINTS\n0\nGamma\n1 1 1\n0 0 0"
        if save_path != None:
            with open(os.path.join(save_path,"KPOINTS"),"w") as f:
                f.write(kpoints)
    elif "kspacing" not in abacus_input:
        kpoints = KptAbacus2Vasp(os.path.join(abacus_path,abacus_input.get("kpt_file","KPT")),None if save_path==None else os.path.join(save_path,"KPOINTS"))
    else:
        kpoints = None

    # 2. write POSCAR
    poscar = stru.write2poscar(None if save_path==None else os.path.join(save_path,"POSCAR"))
    
    # 3. gen POTCAR
    emax = gen_potcar(potcar,stru.get_element(number=False,total=False), None if save_path==None else os.path.join(save_path,"POTCAR"))
    if emax == None:
        print("ERROR: gen POTCAR failed!!!")
    
    # 4. WRITE INCAR
    vasp_input = ParamAbacus2Vasp(abacus_input)
    if "LWAVE" not in vasp_input: vasp_input["LWAVE"] = ".FALSE."
    if "LCHARG" not in vasp_input: vasp_input["LCHARG"] = ".FALSE."
    
    # 4.1 special setting in vasp_setting
    emax_coef = vasp_setting.pop("emax_coef",None)
    
    # 4.2 set ENCUT
    if emax != None and emax_coef != None:
        vasp_input["ENCUT"] = emax * emax_coef
    
    # 4.3 set MAGMOM and deltaspin    
    GenIncarFromStru(stru,vasp_input)
        
    # 4.4 update the vasp_input by vasp_setting    
    if vasp_setting:
        for ik,iv in vasp_setting.items():
            vasp_input[ik.upper()] = iv
    if save_path != None:
        with open(os.path.join(save_path,"INCAR"),"w") as f:
            for key,value in vasp_input.items():
                f.write(f"{key} = {value}\n")
            
    return {
        "incar": vasp_input,  # a dict of incar
        "kpoints": kpoints,     # string of kpoints
        "poscar": poscar        # string of poscar
    }



import unittest,os,shutil,re
from pathlib import Path

from abacustest.myflow import flow,dflowOP,globV
from dflow import (
    Workflow)
import defaultset
from abacustest.myflow import PredftOP,RundftOP,PostdftOP

class TestPredft(unittest.TestCase):
    def setUp(self):
        globV._init()
        globV.set_value("ABBREVIATION",{})
        globV.set_value("PARAM_FNAME","param.json")
        globV.set_value("HOST","test.test")
        self.cwd = os.getcwd()
        self.work_path = Path("testflow")
        os.makedirs("testflow",exist_ok=True)
        os.chdir(self.work_path)
        self.examples = ["a","b","c"]
        self.exatrafiles = ["a.txt","b.txt","c.txt"]
        self.exatrafiles1 = ["a1.txt","b1.txt","c1.txt"]
        self.exatrafiles2 = ["a2.txt","b2.txt","c2.txt"]
        for i in self.examples: 
            os.makedirs(i,exist_ok=True) 
            Path(os.path.join(i,f"{i}.inp")).touch()
        for i in self.exatrafiles: Path(i).touch()
        for i in self.exatrafiles1: Path(i).touch()
        for i in self.exatrafiles2: Path(i).touch()

    def tearDown(self):
        os.chdir(self.cwd)
        if os.path.isdir("testflow"):
            shutil.rmtree("testflow")

    
    def test_predft(self):
        param = {
            "save_path": "result",
            "pre_dft": {
                "example": self.examples,
                "command": "echo 1 > log",
                "image": "python:3.8"
            }   
        }
        globV.set_value("PARAM_CONTEXT",param)
        allstep,stepname,allsave_path = dflowOP.ProduceAllStep(param)
        wf = Workflow(name="abacustest")
        wf.add(allstep)
        wf.submit()
        flow.waitrun(wf,stepname,allsave_path)
        
        expect_dir = ["result", "result/a", "result/b", "result/c"]
        expect_file = ["result/a/log", "result/b/log", "result/c/log",
                       "result/a/a.inp", "result/b/b.inp", "result/c/c.inp"
                       ]
        
        for i in expect_dir:
            self.assertTrue(os.path.isdir(i),f"{i} is not exist!")
        for i in expect_file:
            self.assertTrue(os.path.isfile(i),f"{i} is not exist!")     
    
    def test_rundft(self):
        param = {
            "save_path": "result",
            "run_dft": {
                "example": [["a","b"],"c"],
                "command": "echo run > run.log",
                "image": "python:3.8",
                "extra_files": self.exatrafiles1
            },  
        }
        globV.set_value("PARAM_CONTEXT",param)
        allstep,stepname,allsave_path = dflowOP.ProduceAllStep(param)
        wf = Workflow(name="abacustest")
        wf.add(allstep)
        wf.submit()
        flow.waitrun(wf,stepname,allsave_path)
        
        expect_dir = ["result", "result/a", "result/b", "result/c"]
        expect_file = [
                       "result/a/a.inp", "result/b/b.inp", "result/c/c.inp",
                       "result/a/run.log", "result/b/run.log", "result/c/run.log",
                       "result/a/a1.txt", "result/a/b1.txt", "result/a/c1.txt",
                       "result/b/a1.txt", "result/b/b1.txt", "result/b/c1.txt",
                       "result/c/a1.txt", "result/c/b1.txt", "result/c/c1.txt"
                       ]
        
        for i in expect_dir:
            self.assertTrue(os.path.isdir(i),f"{i} is not exist!")
        for i in expect_file:
            self.assertTrue(os.path.isfile(i),f"{i} is not exist!")

    def test_postdft(self):
        param = {
            "save_path": "result",
            "post_dft": {
                "example": ["a","b","c"],
                "command": "echo post > post.log",
                "image": "python:3.8",
                "extra_files": self.exatrafiles2
            },  
        }
        globV.set_value("PARAM_CONTEXT",param)
        allstep,stepname,allsave_path = dflowOP.ProduceAllStep(param)
        wf = Workflow(name="abacustest")
        wf.add(allstep)
        wf.submit()
        flow.waitrun(wf,stepname,allsave_path)
        
        expect_dir = ["result", "result/a", "result/b", "result/c"]
        expect_file = [
                       "result/a/a.inp", "result/b/b.inp", "result/c/c.inp",
                       "result/post.log", 
                       "result/a2.txt", "result/b2.txt", "result/c2.txt"
                       ]
        
        for i in expect_dir:
            self.assertTrue(os.path.isdir(i),f"{i} is not exist!")
        for i in expect_file:
            self.assertTrue(os.path.isfile(i),f"{i} is not exist!")
        
    def test_predft_rundft_postdft(self):
        param = {
            "save_path": "result",
            "pre_dft": {
                "example": self.examples,
                "command": "mkdir -p d1 d2; echo d1 > workdirs;echo d2 >> workdirs; echo predft > d1/log; cp *.txt d1;cp *.inp d2",
                "image": "python:3.8",
                "extra_files": self.exatrafiles,
                "work_directories_filename": "workdirs"
            } ,
            "run_dft": {
                "command": "echo run > run.log",
                "image": "python:3.8",
                "extra_files": self.exatrafiles1,
                "group_size":2
            },  
            "post_dft": {
                "command": "echo post > post.log",
                "image": "python:3.8",
                "extra_files": self.exatrafiles2,
            },
        }
        globV.set_value("PARAM_CONTEXT",param)
        allstep,stepname,allsave_path = dflowOP.ProduceAllStep(param)
        wf = Workflow(name="abacustest")
        wf.add(allstep)
        wf.submit()
        flow.waitrun(wf,stepname,allsave_path)
        
        expect_dir = ["result", "result/a", "result/b", "result/c",
                      "result/a/d1", "result/b/d1", "result/c/d1",
                      "result/a/d2", "result/b/d2", "result/c/d2",]
        expect_file = [
            "result/a/d1/log", "result/b/d1/log", "result/c/d1/log",
            "result/a/d1/a.txt", "result/a/d1/b.txt", "result/a/d1/c.txt",
            "result/b/d1/a.txt", "result/b/d1/b.txt", "result/b/d1/c.txt",
            "result/c/d1/a.txt", "result/c/d1/b.txt", "result/c/d1/c.txt",
            "result/a/d2/a.inp", "result/b/d2/b.inp", "result/c/d2/c.inp",
            "result/a/d1/run.log", "result/b/d1/run.log", "result/c/d1/run.log",
            "result/a/d2/run.log", "result/b/d2/run.log", "result/c/d2/run.log",
            "result/a/d1/a1.txt", "result/a/d1/b1.txt", "result/a/d1/c1.txt",
            "result/b/d1/a1.txt", "result/b/d1/b1.txt", "result/b/d1/c1.txt",
            "result/c/d1/a1.txt", "result/c/d1/b1.txt", "result/c/d1/c1.txt",
            "result/a/d2/a1.txt", "result/a/d2/b1.txt", "result/a/d2/c1.txt",
            "result/b/d2/a1.txt", "result/b/d2/b1.txt", "result/b/d2/c1.txt",
            "result/c/d2/a1.txt", "result/c/d2/b1.txt", "result/c/d2/c1.txt",
            "result/post.log", "result/a2.txt", "result/b2.txt", "result/c2.txt"
                       ]
        
        for i in expect_dir:
            self.assertTrue(os.path.isdir(i),f"{i} is not exist!")
        for i in expect_file:
            self.assertTrue(os.path.isfile(i),f"{i} is not exist!") 
    
    
    def test_prepare_predft(self):
        os.makedirs("prepare",exist_ok=True)
        for iexample in ["a","b","c"]:
            os.makedirs(os.path.join("prepare",iexample),exist_ok=True)
            Path(os.path.join("prepare",iexample,"STRU")).write_text(defaultset.STRU1)
            Path(os.path.join("prepare",iexample,"INPUT")).write_text(defaultset.INPUT1)
            Path(os.path.join("prepare",iexample,"KPT")).write_text(defaultset.KPT1)
            Path(os.path.join("prepare",iexample,"Si.upf")).write_text("Si.upf")
            Path(os.path.join("prepare",iexample,"Si.orb")).write_text("Si.orb")
        # the new prepared a/b/c should be in current work path, 
        # and has 3*2=6 subdirs named 00000,00001,00002,00003,00004,00005
        param = {
            "save_path": "result",
            "prepare":{
                "example_template":["prepare/a","prepare/b","prepare/c"],
                "mix_input":{
                    "kspacing": [0.1,0.2,0.3],
                    "ks_solver":["cg","dav"]
                }
            },
            "pre_dft": {
                "command": "echo 1 > log",
                "image": "python:3.8"
            }   
        }
        globV.set_value("PARAM_CONTEXT",param)
        allstep,stepname,allsave_path = dflowOP.ProduceAllStep(param)
        wf = Workflow(name="abacustest")
        wf.add(allstep)
        wf.submit()
        flow.waitrun(wf,stepname,allsave_path)
        
        examples = ["a","b","c"]
        sub_dir = ["00000", "00001", "00002", "00003", "00004", "00005"]
        expect_file = ["log", "INPUT", "STRU", "KPT", "Si.upf", "Si.orb"]
        input_set_ref = [[0.1,"cg"],[0.1,"dav"],[0.2,"cg"],[0.2,"dav"],[0.3,"cg"],[0.3,"dav"]]
        input_set_ref.sort()
        
        self.assertTrue(os.path.isdir("result"))
        for iexample in examples:
            input_set = []
            for isubdir in sub_dir:
                self.assertTrue(os.path.isdir(os.path.join("result",iexample,isubdir)))
                for ifile in expect_file:
                    self.assertTrue(os.path.isfile(os.path.join("result",iexample,isubdir,ifile)))
                    
                input_text = Path(os.path.join("result",iexample,isubdir,"INPUT")).read_text()
                kspacing = re.search(r"kspacing\s+0.[1,2,3]",input_text)
                solver = re.search(r"ks_solver\s+(cg|dav)",input_text)
                self.assertTrue(kspacing)
                self.assertTrue(solver)
                input_set.append([float(kspacing.group().split()[1]),solver.group().split()[1]])
            input_set.sort()
            self.assertEqual(input_set,input_set_ref)
                
        
        
        
        


POTENTIAL = {"H":"GTH-BLYP-q1", "He":"GTH-BLYP-q2", "Li":"GTH-BLYP-q3", "Be":"GTH-BLYP-q4", "B":"GTH-BLYP-q3", "C":"GTH-BLYP-q4", "N":"GTH-BLYP-q5", "O":"GTH-BLYP-q6", "F":"GTH-BLYP-q7", "Ne":"GTH-BLYP-q8", "Na":"GTH-BLYP-q1", "Mg":"GTH-BLYP-q10", "Al":"GTH-BLYP-q3", "Si":"GTH-BLYP-q4", "P":"GTH-BLYP-q5", "S":"GTH-BLYP-q6", "Cl":"GTH-BLYP-q7", "Ar":"GTH-BLYP-q8", "K":"GTH-BLYP-q9", "Ca":"GTH-BLYP-q10", "Sc":"GTH-BLYP-q11", "Ti":"GTH-BLYP-q12", "V":"GTH-BLYP-q13", "Cr":"GTH-BLYP-q14", "Mn":"GTH-BLYP-q15", "Fe":"GTH-BLYP-q16", "Co":"GTH-BLYP-q17", "Ni":"GTH-BLYP-q18", "Cu":"GTH-BLYP-q11", "Zn":"GTH-BLYP-q12", "Ga":"GTH-BLYP-q13", "Ge":"GTH-BLYP-q4", "As":"GTH-BLYP-q5", "Se":"GTH-BLYP-q6", "Br":"GTH-BLYP-q7", "Kr":"GTH-BLYP-q8", "Rb":"GTH-BLYP-q9", "Sr":"GTH-BLYP-q10", "Y":"GTH-BLYP-q11", "Zr":"GTH-BLYP-q12", "Mo":"GTH-BLYP-q14", "Ru":"GTH-BLYP-q16", "Rh":"GTH-BLYP-q17", "Pd":"GTH-BLYP-q18", "Ag":"GTH-BLYP-q11", "In":"GTH-BLYP-q13", "Sb":"GTH-BLYP-q5", "Te":"GTH-BLYP-q6", "I":"GTH-BLYP-q7", "Xe":"GTH-BLYP-q8", "Cs":"GTH-BLYP-q9", "Ba":"GTH-BLYP-q10", "Ce":"GTH-BLYP-q12", "Gd":"GTH-BLYP-q18", "W":"GTH-BLYP-q14", "Au":"GTH-BLYP-q11", "Pb":"GTH-BLYP-q4", "Bi":"GTH-BLYP-q15"}
BASIS = {"H":"DZVP-MOLOPT-SR-GTH-q1", "He":"DZVP-MOLOPT-SR-GTH-q2", "Li":"DZVP-MOLOPT-SR-GTH-q3", "Be":"DZVP-MOLOPT-SR-GTH-q4", "B":"DZVP-MOLOPT-SR-GTH-q3", "C":"DZVP-MOLOPT-SR-GTH-q4", "N":"DZVP-MOLOPT-SR-GTH-q5", "O":"DZVP-MOLOPT-SR-GTH-q6", "F":"DZVP-MOLOPT-SR-GTH-q7", "Ne":"DZVP-MOLOPT-SR-GTH-q8", "Na":"DZVP-MOLOPT-SR-GTH-q9", "Mg":"DZVP-MOLOPT-SR-GTH-q10", "Al":"DZVP-MOLOPT-SR-GTH-q3", "Si":"DZVP-MOLOPT-SR-GTH-q4", "P":"DZVP-MOLOPT-SR-GTH-q5", "S":"DZVP-MOLOPT-SR-GTH-q6", "Cl":"DZVP-MOLOPT-SR-GTH-q7", "Ar":"DZVP-MOLOPT-SR-GTH-q8", "K":"DZVP-MOLOPT-SR-GTH-q9", "Ca":"DZVP-MOLOPT-SR-GTH-q10", "Sc":"DZVP-MOLOPT-SR-GTH-q11", "Ti":"DZVP-MOLOPT-SR-GTH-q12", "V":"DZVP-MOLOPT-SR-GTH-q13", "Cr":"DZVP-MOLOPT-SR-GTH-q14", "Mn":"DZVP-MOLOPT-SR-GTH-q15", "Fe":"DZVP-MOLOPT-SR-GTH-q16", "Co":"DZVP-MOLOPT-SR-GTH-q17", "Ni":"DZVP-MOLOPT-SR-GTH-q18", "Cu":"DZVP-MOLOPT-SR-GTH-q11", "Zn":"DZVP-MOLOPT-SR-GTH-q12", "Ga":"DZVP-MOLOPT-SR-GTH-q13", "Ge":"DZVP-MOLOPT-SR-GTH-q4", "As":"DZVP-MOLOPT-SR-GTH-q5", "Se":"DZVP-MOLOPT-SR-GTH-q6", "Br":"DZVP-MOLOPT-SR-GTH-q7", "Kr":"DZVP-MOLOPT-SR-GTH-q8", "Rb":"DZVP-MOLOPT-SR-GTH-q9", "Sr":"DZVP-MOLOPT-SR-GTH-q10", "Y":"DZVP-MOLOPT-SR-GTH-q11", "Zr":"DZVP-MOLOPT-SR-GTH-q12", "Nb":"DZVP-MOLOPT-SR-GTH-q13", "Mo":"DZVP-MOLOPT-SR-GTH-q14", "Tc":"DZVP-MOLOPT-SR-GTH-q15", "Ru":"DZVP-MOLOPT-SR-GTH-q16", "Rh":"DZVP-MOLOPT-SR-GTH-q17", "Pd":"DZVP-MOLOPT-SR-GTH-q18", "Ag":"DZVP-MOLOPT-SR-GTH-q11", "Cd":"DZVP-MOLOPT-SR-GTH-q12", "In":"DZVP-MOLOPT-SR-GTH-q13", "Sn":"DZVP-MOLOPT-SR-GTH-q4", "Sb":"DZVP-MOLOPT-SR-GTH-q5", "Te":"DZVP-MOLOPT-SR-GTH-q6", "I":"DZVP-MOLOPT-SR-GTH-q7", "Xe":"DZVP-MOLOPT-SR-GTH-q8", "Cs":"DZVP-MOLOPT-SR-GTH-q9", "Ba":"DZVP-MOLOPT-SR-GTH-q10", "Hf":"DZVP-MOLOPT-SR-GTH-q12", "Ta":"DZVP-MOLOPT-SR-GTH-q13", "W":"DZVP-MOLOPT-SR-GTH-q14", "Re":"DZVP-MOLOPT-SR-GTH-q15", "Os":"DZVP-MOLOPT-SR-GTH-q16", "Ir":"DZVP-MOLOPT-SR-GTH-q17", "Pt":"DZVP-MOLOPT-SR-GTH-q18", "Au":"DZVP-MOLOPT-SR-GTH-q11", "Hg":"DZVP-MOLOPT-SR-GTH-q12", "Tl":"DZVP-MOLOPT-SR-GTH-q13", "Pb":"DZVP-MOLOPT-SR-GTH-q4", "Bi":"DZVP-MOLOPT-SR-GTH-q5", "Po":"DZVP-MOLOPT-SR-GTH-q6", "At":"DZVP-MOLOPT-SR-GTH-q7", "Rn":"DZVP-MOLOPT-SR-GTH-q8"}



class Model:
    '''
    Base class for all models
    
    Some specific workflow will be implemented in the subclass.
    Such as EOS, BAND, etc.
    
    The model should have the following methods:
    '''
    @staticmethod
    def model_name():
        '''
        Name of the model, which will be used as the subcommand
        '''
        return "Model"
    
    @staticmethod
    def description():
        '''
        Description of the model
        '''
        return ""
    
    @staticmethod
    def add_args(parser):
        '''
        Add arguments for the model
        The arguments can not be command, model, modelcommand 
        '''
        pass
    
    def run(self,params):
        '''
        Parse the parameters and run the model
        '''
        pass
    
    @staticmethod
    def prepare_args(parser):
        '''
        Add arguments for the prepare subcommand
        The arguments can not be command, model, modelcommand '''
        pass
    
    def run_prepare(self,params):
        '''
        Parse the parameters and run the prepare process.
        Usually, this step will generate the input files for abacustest submit.
        '''
        pass
    
    @staticmethod
    def postprocess_args(parser):
        '''
        Add arguments for the postprocess subcommand
        The arguments can not be command, model, modelcommand'''
        pass

    def run_postprocess(self,params):
        '''
        Parse the parameters and run the postprocess process'''
        pass


import traceback,inspect

class Result:
    def __init__(self):
        self._PARAM_VALUE={}
    
    @classmethod
    def register(cls,**key):
        def aa(method):
            
            for ikey,descript in key.items():
                #print("register",ikey)
                if ikey in cls._PARAM_DIC:
                    method_org = cls._PARAM_DIC[ikey][0].__name__
                    print("WARNING: '%s' has been defined in %s(), modify to %s()" % 
                          (ikey,method_org,method.__name__))
                cls._PARAM_DIC[ikey] = (method,descript)
                #print(cls)

                #for line in inspect.getsourcelines(method):
                #    if ("self.[\'" in line and "\']" in line) or \
                #        ("self.[\"" in line and "\"]" in line):
                            
            return method
        return aa

    def __setitem__(self,key,value):
        if key not in self._PARAM_DIC:
            print("WARNING: the method to catch '%s' is not registered, but is doing the assignment" % key)
            print(inspect.currentframe().f_back)

        try:
            value_org = self._PARAM_VALUE[key]
            if value_org != value:
                print("WARNING: value of '%s' has been catched, but value is different:" % key,value_org,value)
        except:
            self._PARAM_VALUE[key] = value

    def __getitem__(self,key):
        try:
            return self._PARAM_VALUE[key]
        except:
            pass

        try:
            func = self._PARAM_DIC[key][0]
        except:
            print("ERROR: no method to catch value of '%s', or the method is not registered!" % key)
            return None
        
        func_source_file = inspect.getsource(func)
        fback_source_file = inspect.getsource(inspect.currentframe().f_back)
        if func_source_file == fback_source_file:
            framinfo = inspect.getframeinfo(inspect.currentframe().f_back)
            print("WARNING: try to catch the value of '%s' by %s()" % (key,func.__qualname__))
            print("  This is a recursion call, and should be avoid!")
            print("  File:'%s', line %d, %s()" % framinfo[0:3])
            print("%s" % framinfo[3][0])
            return None

        try:
            func(self)
        except:
            print("ERROR: excecute function %s() failed, skip it!" % func.__qualname__)
            traceback.print_exc()
        
        try:
            return self._PARAM_VALUE[key]
        except:
            print("WARNING: try to catch the value of '%s' by %s(), but failed" % 
                    (key,self._PARAM_DIC[key][0].__qualname__))
            return None

    @classmethod
    def AllMethod(cls):
        #return a dict, whose key is the parameter name, 
        #and value is a tuple of functionname, filename, and the description
        paramdic = {}
        for key,value in cls._PARAM_DIC.items():
            filename = value[0].__code__.co_filename
            value = ("%s()"% (value[0].__qualname__),filename,value[1])
            paramdic[key] = value
        return paramdic
    
    def AllParamValue(self):
        return self._PARAM_VALUE




from dp.launching.typing.basic import BaseModel, Int, String
from dp.launching.typing import (
    Set,
    Boolean,
    Field,
    BenchmarkLabels,
    InputFilePath
)
from dp.launching.report import Report

from . import comm_class,comm_func,get_aim_data,comm_class_exampleSource
import json,traceback,datetime,os,re,pickle
from abacustest.outresult import Table2FeishuInteractive
from dp.launching.report import ChartReportElement,ReportSection,AutoReportElement
import dp.launching.typing.addon.ui as ui
from dp.launching.typing.addon.sysmbol import Equal

def transfer_tag(old_tag):
    """
    transfer:
        "aim_tag":{
        "intel-cg": ["benchmark-profile-pw-cg","benchmark-schedule-intel-cg"],
        "intel-dav": ["benchmark-profile-pw-dav","benchmark-schedule-intel-dav"]}
    to:
        "aim_tag":{
        "benchmark-profile-pw-cg": "intel-cg",
        "benchmark-schedule-intel-cg": "intel-cg",
        "benchmark-profile-pw-dav": "intel-dav",
        "benchmark-schedule-intel-dav": "intel-dav"}
    """
    new_tag = {}
    for k,v in old_tag.items():
        for i in v:
            new_tag[i] = k
    return new_tag

def clean_metrics_coef(profile,metrics,metrics_coef):
    """
    return {(iprofile,imetrics):coef}
    """
    new_metrics_coef = {}
    for i in profile:
        for j in metrics:
            if j not in metrics_coef:
                new_metrics_coef[(i,j)] = 1.0
            elif isinstance(metrics_coef[j],dict):
                new_metrics_coef[(i,j)] = metrics_coef[j].get(i,1.0)
            else:
                new_metrics_coef[(i,j)] = metrics_coef[j]
    return new_metrics_coef

def get_profile_value(allruninfos,profile,aim_tag,metrics,metrics_coef,metrics_name,token):
    '''
    allruninfos = {
        "run_name": run["props"]["name"],
        "experiment_name": run["props"]["experiment"]["name"],
        "tags": [i["name"] for i in run["props"]["tags"]],
        "creation_time": run["props"]["creation_time"],
        "end_time": run["props"]["end_time"],
        "metric": metric
    },

    metric = {metric_name: value}
    value is a list

    allvalues = {
        profile_i: {metric_i:[[create_time,metric_values,runname],...],}
    }
    profile_i and metric_i are the names in profile and metrics, which are the same as the name in final table.
    metric_values = value * coef
    '''
    new_tag = transfer_tag(aim_tag)
    new_metrics_coef = clean_metrics_coef(profile,metrics,metrics_coef)
    
    allvalues = {}
    for i in profile: allvalues[i] = {}
    for run in allruninfos:
        #only get run_name endswith ".summary"
        if not run["run_name"].endswith(".summary"):
            continue
        runname = run["run_name"].split(".")[1]
        #if runname.startswith("sched"):
        #    url = "https://labs.dp.tech/projects/abacustest/?request=GET%3A%2Fapplications%2Fabacustest%2Fjobs%2F" + runname
        #else:
        #    url = "https://benchmark.mlops.dp.tech/?request=GET%3A%2Fprojects%2Fabacustest%2Fruns%2F" + runname
        # url = "https://labs.dp.tech/projects/abacustest/?request=GET%3A%2Fapplications%2Fabacustest%2Fjobs%2F" + runname
        url = "https://app.bohrium.dp.tech/abacustest/?request=GET%3A%2Fapplications%2Fabacustest%2Fjobs%2F" + runname
        tags = run["tags"] # tags in AIM
        create_time = datetime.datetime.utcfromtimestamp(run["creation_time"]+8*3600)

        # loop AIM tags, and if the tag is in new_tag, then this run is needed
        for itag in tags:
            if itag in new_tag and new_tag[itag] in allvalues: # check if the this run is needed
                iprofile = new_tag[itag]
                needed_run = False
                metric_value_number_max = 1
                for imetric in metrics: # loop needed metrics
                    for metric_in_aim in metrics_name.get(imetric,[imetric]):  # metric_in_aim is the name of needed metrics in AIM 
                        if metric_in_aim in run["metric"]:  #check if this run has the needed metrics
                            needed_run = True
                            if imetric not in allvalues[iprofile]:
                                allvalues[iprofile][imetric] = []
                            allvalues[iprofile][imetric].append([create_time,
                                                                 run["metric"][metric_in_aim][0] * new_metrics_coef[(iprofile,imetric)],
                                                                 url, run["run_hash"]]) #the last one should be version, store hash now
                            metric_value_number_max = len(allvalues[iprofile][imetric])
                            break
                
                #consider the case that this run has partial of needed metrics, we need to add None for the missing metrics
                if needed_run:
                    for imetric in metrics:
                        if imetric not in allvalues[iprofile]:
                            allvalues[iprofile][imetric] = []
                        while len(allvalues[iprofile][imetric]) < metric_value_number_max:
                            allvalues[iprofile][imetric].append([create_time,None,url,run["run_hash"]])
                break
    versions = {}
    for k,v in allvalues.items():
        hash = None
        for ik,iv in v.items():
            iv.sort()
            hash = iv[-1][-1]
            #find the version of last iv
            if hash not in versions:
                versions[hash] = get_aim_data.get_version(hash,token)
            iv[-1][-1] = None if not versions[hash] else versions[hash][0]
            #set version to be None, except for the last iv
            for i in iv[:-1]:
                i[-1] = None
    return allvalues

def color_value(value,allvalues,color_method,color_list=["green","red"]):
    '''
    if value is not float or int, then return None
    
    allvalues should be a list of history value.
    
    metrics_color define how to color the value, should be a dict, where key is the metric name, value is color stragegy: 
    1. a string, which is a python format string that can be eval to a bool value. For example: "x >= 1.0" means if x >= 1.0, then color it to green else red.
    2. a list of a list of two string, each sublist has two string,the first one is the color method, the second one is the color value. 
        And if the value not meet all the color method, then use the last color value in color_list.
    such as: "x>=1.0" or [["x>=1.0","green"],["x>=0.5","yellow"],["x>=0.0","red"]]
        
    The string can be some special case: 
        "SIGMA 10 1" means use the last 10 values to calculate the mean and std, then color the value to green if it is in the range of mean -1*std to 1*std, else red.
        "SIGMAU 10 1" means use the last 10 values to calculate the mean and std, then color the value to green if it is larger than mean to 1*std, else red.
        "SIGMAD 10 1" means use the last 10 values to calculate the mean and std, then color the value to green if it is smaller than mean to 1*std, else red.
    
    color_list has two values, the first one is the color for True, the second one is the color for False.
    '''
    if not isinstance(value,(int,float)):
        return None
    if color_method == None:
        return None
    if isinstance(color_method,str):
        color_method = [[color_method,color_list[0]]]
    print("value,color_method: ",value,color_method)       
    for icolor,color in enumerate(color_method):
        if color[0].startswith("SIGMA"):
            sigma = color[0].split()
            sigma_method = sigma[0]
            sigma_number = int(sigma[1])
            sigma_range = float(sigma[2])
            if len(allvalues) < sigma_number:
                real_allvalues = [j for j in allvalues if j != None]
            else:
                real_allvalues = []
                for i in range(sigma_number):
                    if allvalues[-1-i] != None:
                        real_allvalues.append(allvalues[-1-i])
                    if len(real_allvalues) == sigma_number:
                        break
            if len(real_allvalues) == 0:
                return color[1]  
            mean = sum(real_allvalues)/len(real_allvalues)
            std = (sum([(i-mean)**2 for i in real_allvalues])/len(real_allvalues))**0.5
            print("value,mean,std,sigma: ",value,mean,std,sigma)
            if (sigma_method == "SIGMA" and (mean - sigma_range*std <= value and value <= mean + sigma_range*std)) or \
                (sigma_method == "SIGMAU" and value >= mean + sigma_range*std) or \
                (sigma_method == "SIGMAD" and value <= mean - sigma_range*std):
                    return color[1]
        else:
            x = value
            if eval(color[0]):
                return color[1]
    return color_list[-1]

def produce_outtable(allvalues,profile,metrics,digit,metrics_color,color_list=["green","red"]):
    '''
    outtable = [
        ["profile","date","profile_link",metrics_value],
        ititle1,
        ["profile_name1",value1,value2,...],
        ...,
        ititle2,
        ["profile_name2",value1,value2,...],
        ...]
    profile = [{"title":,"profiles":[]},...]
    metrics_value = a string for the final display
    This function will collect the last value, which may not be today
    
    metrics_color is a dict, which key is the metrics name and value is the color method, or the value is a dict that key is profile name and value is color method.
    for a dict case, can use "000" as the key to define the default color method.
    such as:
    {
        "NormalEnd_ratio": "x>=1.0",
        "Converge_ratio": "x>=1.0",
        "SCFConverge Score": {"intel-cg": "x>=100", "intel-dav": "x>=100", "intel-elpa": "x>=100", "intel-scalapack": "x>=100", "000": "x>=100"},
    }
    '''
    
    outtable = [["profile","date","profile_link","version(commit)"]]
    for ii in metrics:
        outtable[0].append(ii)
        
    for sub_profile in profile:
        outtable.append([sub_profile["title"]])
        for iprofile in sub_profile["profiles"]:
            if iprofile not in allvalues:
                continue
            profile_value = allvalues[iprofile]

            outtable.append([iprofile,None,None,None])
            not_set_date = True
            for ik in metrics:
                
                color_method = None
                
                if ik in metrics_color:
                    if isinstance(metrics_color[ik],dict):
                        if iprofile in metrics_color[ik]:
                            color_method = metrics_color[ik][iprofile]
                        elif "000" in metrics_color[ik]:
                            color_method = metrics_color[ik][0]
                    else:
                        color_method = metrics_color[ik]
                        
                ivalue = profile_value.get(ik)
                if ivalue:
                    if not_set_date:
                        outtable[-1][1] = ivalue[-1][0].strftime('%Y/%m/%d %H:%M')
                        outtable[-1][2] = ivalue[-1][2]
                        outtable[-1][3] = ivalue[-1][3]
                        not_set_date = False

                    value0 = ivalue[-1][1]
                    if value0 == None:
                        value = "--"
                    else:
                        value_abs = ("%." + "%df"%digit.get(ik,0)) % float(value0)
                        value = value_abs + "[--]"
                        if len(ivalue) > 1:
                            value1 = ivalue[-2][1]
                            if value1 != 0 and isinstance(value0,(int,float)) and isinstance(value1,(int,float)):
                                value_rel = ("%" + ".2f" + "%" + "%") % ((value0 - value1)*100/value1)
                                if (value0 - value1) >= 0:
                                    value_rel = "+" + value_rel
                                value = value_abs + f"[{value_rel}]"
                                
                    allvalue = [i[1] for i in ivalue[:-1] if i[1] != None]
                else:
                    allvalue = []
                    value = "--"
                    value0 = None
                print("iprofile,ik,color_method,value:",iprofile,ik,color_method,value)    
                if color_method != None and value != "--":
                    color = color_value(value0,allvalue,color_method,color_list)
                    if color == None:
                        color = color_list[-1]
                    value = "<font color='%s'>" % color + value + "</font>"

                outtable[-1].append(value)
    return outtable
        
def send_to_feishu(outtable,webhook,comment):
    today = datetime.datetime.now().strftime('%Y/%m/%d')
    new_table = [["profile"] + [i for i in outtable[0][4:]]]  #table head

    version_list = []
    profile_list = []
    for itable in outtable[1:]:
        if len(itable) == 1:
            new_table.append(itable)
            continue
        iprofile = itable[0]
        date0 = itable[1]
        link = itable[2]
        profile_list.append(iprofile)
        today_data = False
        if date0 != None and date0.split()[0] == today:
            today_data = True
            version_list.append(itable[3])
            if link != None:
                iprofile = f"[{iprofile}]({link})"

        inew_table = [iprofile]
        has_today_data = False
        for value in itable[4:]:
            if today_data: 
                inew_table.append(value)
                has_today_data = True
            else:
                inew_table.append("--")
        if has_today_data:
            new_table.append(inew_table)

    new_comment = ""
    if len(set(version_list)) == 1 and version_list[0] != None:
        new_comment += f"version: {version_list[0]}\n"
    elif len(set(version_list)) > 1:
        version_list_remove_none = set(version_list) - set([None])
        if len(version_list_remove_none) == 1:
            new_comment += f"version: {list(version_list_remove_none)[0]}\n"
        else:
            new_comment += f"version: {version_list}\n"
        
    
    Table2FeishuInteractive(new_table,webhook,title="abacus performance test %s" % today,comment=new_comment + comment)
    return outtable

def produce_html(outtable,comment):
    import copy
    new_table = [["profile"] + [i for i in outtable[0][4:]] + ["test_date","version"]]  #table head

    for itable in outtable[1:]:
        if len(itable) == 1:
            new_table.append(itable)
            continue
        iprofile = itable[0]
        date0 = "--" if not itable[1] else itable[1].split()[0]
        link = itable[2]
        version = itable[3]
        if link != None:
            iprofile = f"<a href=\"{link}\" target=\"detail\">{iprofile}</a>"

        new_table.append([iprofile])
        for value in itable[4:]:
            new_table[-1].append(value)
        new_table[-1].append(date0)
        
        if version != None:
            pattern = r"v([\d.]+)\((\w+)\s\((.+)\)\)"
            result = re.match(pattern,str(version))
            if result:
                commit = result.group(2)
                github_link = "https://github.com/deepmodeling/abacus-develop/commit/" + commit
                version = f"<a href=\"{github_link}\" target=\"detail\">{version}</a>"
        new_table[-1].append(str(version))

    new_table_nolink = copy.deepcopy(new_table)
    for i in range(1,len(new_table_nolink)):
        if "<a href=" in new_table_nolink[i][0]:
            profile_name = new_table_nolink[i][0].split(">")[1].split("<")[0]
            new_table_nolink[i][0] = profile_name
    
    head_set = """
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <style>
        body {
            text-align: left;
            font-family: Arial, sans-serif;
        }
        table {
            text-align: center;
        }
        .description {
            font-family: Arial, sans-serif;
            text-align: left;
            display: inline-block;
            font-size: 16px;
        }
    </style>
</head>
"""
    table_set = comm_func.produce_html_table(new_table) 
    table_set_nolink = comm_func.produce_html_table(new_table_nolink)
    description_set = "<div class=\"description\"><pre>" + comment + "\nClick profile to check the detail launching run.</pre></div>"

    html = "<html>"  + head_set + "<body>" + table_set + description_set + '</body></html>'
    #html_nolink = "<html>"  + head_set + "<body>" + table_set_nolink + '</body></html>'
    # As the results are on labs, which can be accessed by everyone, we do not need to hide the link, only remove the description
    html_nolink = "<html>"  + head_set + "<body>" + table_set + '</body></html>'

    return html,html_nolink

def echart_report(allvalues):
    #transfer the allvalues format
    
    chart_section = []
    for iprofile,profile_value in allvalues.items():
        metric_name = []
        metric_value = []
        for imetric,imetric_value in profile_value.items():
            ivalue = []
            for ii in imetric_value:
                ivalue.append([ii[0].strftime('%Y/%m/%d\n%H:%M'),ii[1]])
                #ivalue.append([ii[0],ii[1][0]*coef])
            metric_name.append(imetric)
            metric_value.append(ivalue)
        if len(metric_name) == 0:
            continue
        nmetrics = len(metric_name)
        inter = 5
        height = (0.9 - inter/100.0 * (nmetrics-1))/nmetrics*100
        interv = height + inter

        option = {
            "title": {
                "text": iprofile,
                "left": 'center',
                "top": 0
            },
            "legend": {
                "data": metric_name,
                "top":"10%"
            },
            "toolbox": {
                "feature": {
                    "dataZoom": {},
                    "dataView": {"readOnly": False},
                    "magicType": {"type": ['line', 'bar']},
                    "restore": {},
                    "saveAsImage": {}
                }
            },
            "tooltip": {
                "trigger": 'axis',
                "axisPointer": {
                    "type": 'cross',
                    "animation": False,
                    "label": {
                        "backgroundColor": '#505765'
                    }
                }
            },
            "dataZoom": 
                {
                    "show": True,
                    "realtime": True,
                    "start":80,
                    "end":100,
                    "xAxisIndex":list(range(nmetrics)),
                    "left":"5%",
                    "right":"5%"
                }
            ,
            "grid": [{"left": f"{i*interv+5}%","right":f"{(nmetrics-i-1)*interv+5}%"} for i in range(nmetrics)],
            "xAxis": [{"type": "category", "gridIndex": i} for i in range(nmetrics)],
            "yAxis": [{"gridIndex": i,"scale": True,"name":metric_name[i],"nameLocation":"middle","axisLabel":{"inside":True}} for i in range(nmetrics)],
            "series": [
                {
                    "name": metric_name[i],
                    "type": 'line',
                    "xAxisIndex": i,
                    "yAxisIndex": i,
                    "data": metric_value[i],
                    "markLine": "markLineOpt"
                } for i in range(nmetrics)
            ]
        }
        chart_section.append(ChartReportElement(options=option,title=iprofile))

    return ReportSection(title="metrics chart",elements=chart_section,ncols=1)

def plot_chart(values,minmax):
    '''
    values = {
        profile_name: [],
        metric_name:[],
        title:,
        profile1:{
            create_time:[value1,value2,...],
            metric_name1:[value1,value2,...],
            metric_name2:[value1,value2,...]
            ...
        },
        profile2:{
            create_time:[value1,value2,...],
            metric_name1:[value1,value2,...],
            metric_name2:[value1,value2,...]
        }}
    minmax = {
        metric_name1:[min,max],
        metric_name2:[min,max],
        ...}
        
    return a line chart
    '''
    from pyecharts import options as opts
    from pyecharts.charts import Line,Grid
    color_list = [
    "#c23531", "#2f4554", "#61a0a8", "#d48265", "#91c7ae",
    "#749f83", "#ca8622", "#bda29a", "#6e7074", "#546570",
    "#c4ccd3", "#f05b72", "#ef5b9c", "#f47920", "#905a3d",
    "#fab27b", "#2a5caa", "#444693", "#726930", "#b2d235"]
    '''
    1. "#c23531" - 砖红色
    2. "#2f4554" - 深蓝灰色
    3. "#61a0a8" - 青绿色
    4. "#d48265" - 淡赭色
    5. "#91c7ae" - 草绿色
    6. "#749f83" - 绿灰色
    7. "#ca8622" - 金黄色
    8. "#bda29a" - 灰褐色
    9. "#6e7074" - 深灰色
    10. "#546570" - 蓝灰色
    11. "#c4ccd3" - 亮灰色
    12. "#f05b72" - 水粉红色
    13. "#ef5b9c" - 粉红色
    14. "#f47920" - 橙色
    15. "#905a3d" - 棕色
    16. "#fab27b" - 杏色
    17. "#2a5caa" - 蓝色
    18. "#444693" - 深蓝色
    19. "#726930" - 橄榄色
    20. "#b2d235" - 黄绿色
    '''
    name_locations = ["start","end"]
    
    profile_name = values["profile_name"]
    metric_name = values["metric_name"]
    final_chart = Line()
    line_idx = 0
    width = 1000
    height = 400
    grid = Grid(init_opts=opts.InitOpts(width=f"{width}px", height=f"{height}px"))

    
    for profile_idx in range(len(profile_name)):
        iprofile = profile_name[profile_idx]
        line_chart = Line()
        line_chart.add_xaxis(xaxis_data=values[iprofile]["create_time"])
        for metric_idx in range(len(metric_name)):
            imetric = metric_name[metric_idx]
            if len(profile_name) == 1:
                legend_name = imetric
            elif len(metric_name) == 1:
                legend_name = iprofile
            else:
                legend_name = iprofile + "/" + imetric 
            
            if metric_idx > 0:
                line_chart.extend_axis(
                    yaxis=opts.AxisOpts(
                        name=imetric,
                        name_location=name_locations[metric_idx % len(name_locations)],
                        type_="value",
                        min_=minmax[imetric][0],
                        max_=minmax[imetric][1],
                        position="right",
                        offset=80*(metric_idx-1),
                        axisline_opts=opts.AxisLineOpts(linestyle_opts=opts.LineStyleOpts(color=color_list[line_idx % len(color_list)]),),
                        axistick_opts=opts.AxisTickOpts(is_align_with_label=True),
                    ))
            line_chart.add_yaxis(
                legend_name,
                values[iprofile][imetric],
                xaxis_index=0,
                yaxis_index=metric_idx,
                label_opts=opts.LabelOpts(is_show=False),
                color=color_list[ line_idx % len(color_list)]
            )
            line_idx += 1
        
        if profile_idx == 0:
            line_chart.set_global_opts(
                xaxis_opts=opts.AxisOpts(type_="time", boundary_gap=False),
                yaxis_opts=opts.AxisOpts(name=metric_name[0],
                                         min_=minmax[metric_name[0]][0],
                                         max_=minmax[metric_name[0]][1],
                                         #is_scale = True,
                                         name_location="middle",
                                         name_gap=30,
                                         axisline_opts=opts.AxisLineOpts(
                    linestyle_opts=opts.LineStyleOpts(color=color_list[0])),
                    axistick_opts=opts.AxisTickOpts(is_align_with_label=True),
                ),
                datazoom_opts=[opts.DataZoomOpts(type_="slider", range_start=80, range_end=100),
                               opts.DataZoomOpts(type_="inside", range_start=80, range_end=100),],
                tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
                legend_opts=opts.LegendOpts(is_show=True, pos_top="30"),
                title_opts=opts.TitleOpts(title=values["title"], pos_left="center", pos_top="top"),
                toolbox_opts=opts.ToolboxOpts(feature=opts.ToolBoxFeatureOpts(save_as_image=opts.ToolBoxFeatureSaveAsImageOpts(type_="png", background_color="white"),
                                                                              data_zoom=opts.ToolBoxFeatureDataZoomOpts(),
                                                                              restore=opts.ToolBoxFeatureRestoreOpts(),
                                                                              data_view=opts.ToolBoxFeatureDataViewOpts(),
                                                                              magic_type=opts.ToolBoxFeatureMagicTypeOpts(type_=["line", "bar"]),
                                                                              brush=opts.ToolBoxFeatureBrushOpts(type_="rect")),
                                              pos_left="50",
                                              pos_top="60"),
            )
            
        if profile_idx == 0:
            final_chart = line_chart
        else:
            final_chart.overlap(line_chart)
    
    
    right_shift = 100 if len(metric_name) < 3 else 100+80*(len(metric_name)-2)
    grid.add(
        final_chart,
        grid_opts=opts.GridOpts(
            pos_left="50px", pos_top="90px", pos_right=f"{right_shift}px", pos_bottom="70px"),
        is_control_axis_index=True
    )
       
    return grid

def echart_html(allvalues,value_range,setting,filename="all.html"):
    '''
    setting = [
        {
            "profiles": ["intel-cg","intel-dav",...],
            "metrics": ["NormalEndRatio","ConvergeRatio",...],
            "title":""
        },
        {
            "profiles": ["intel-cg","intel-dav",...],
            "metrics": ["SCFConverge Score","Performance Score",...],
            "title":""   
        }
    ]
    '''
    from pyecharts import options as opts
    from pyecharts.charts import Line,Grid,Page
    
    pre_path = os.path.split(filename)[0]
    page = Page(layout=Page.SimplePageLayout)
    
    chart_idx = 0
    for iset in setting:
        profiles = iset["profiles"]
        metrics = iset["metrics"]
        title = iset["title"]
        if title.strip() == "":
            title = "metrics chart{chart_idx}"
        
        profile_name = []
        metric_name = []
        chart_values = {}
        for iprofile in profiles:
            if iprofile not in allvalues:
                continue
            profile_value = {}
            profile_has_value = False
            for imetrics in metrics:
                if imetrics not in allvalues[iprofile]:
                    continue

                x = []
                y = []
                all_none = True
                for ii in allvalues[iprofile][imetrics]:
                    y.append(ii[1])
                    x.append(ii[0])
                    if ii[1] != None:
                        all_none = False
                        
                if all_none:
                    continue
                profile_value["create_time"] = x
                profile_value[imetrics] = y
                profile_has_value = True
                if imetrics not in metric_name:
                    metric_name.append(imetrics)
                    
            if profile_has_value:
                profile_name.append(iprofile)
                chart_values[iprofile] = profile_value
        
        if len(profile_name) == 0 or len(metric_name) == 0:
            continue
        chart_idx += 1
        
        # get min and max of each metric
        minmax = {}
        for imetric in metric_name:
            if value_range.get(imetric,None) != None:
                minmax[imetric] = value_range[imetric]
            else:
                y_no_none = []
                for iprofile in profile_name:
                    if imetric not in chart_values[iprofile]:
                        chart_values[iprofile][imetric] = [None for i in chart_values[iprofile]["create_time"]]
                    else:
                        y_no_none += [ii for ii in chart_values[iprofile][imetric] if ii is not None]
                minmax[imetric] = [min(y_no_none) - 1,max(y_no_none) + 1]
                  
        chart_values["profile_name"] = profile_name
        chart_values["metric_name"] = metric_name
        chart_values["title"] = title      

        chart = plot_chart(chart_values,minmax)
        chart_name = "_".join(title.split())
        chart.render(f"{os.path.join(pre_path,chart_name)}.html")
        page.add(chart)
    page.render(filename)
 
class Summary(BaseModel):
    feishu_webhook:  String = Field(default = None,
                                    title="FeiShu Webhook")
    AIM_TOKEN: String = Field(title="AIM tracking token")
    setting: String = Field(default = None,title="setting json string")
    setting_file:InputFilePath = Field(default = None,
                                        title="Upload setting file",
                                        st_kwargs_type = ["json"], 
                                        description="Please upload the setting file or enter the setting information in latter 'setting' section.",
                                        description_type="markdown")
    Config_dflow_labels: BenchmarkLabels
    setting_file_name: String = Field(default = None,description="If use setting file in dataset, please enter the setting file name here")
    #experiment: String = Field(title="abacustest/benchmark")
    #experiment_id: String = Field(title="7ab4e46a-43fb-440a-828d-4fbdef5b4709")

group1 = ui.Group("if_load_data","test")

@group1
@ui.Visible(Summary,("setting_file"),Equal,(True))
class LoadData(BaseModel):
    if_load_data: Boolean = Field(default = False)

class SummaryModel(LoadData,
                   comm_class.ConfigSetGithub,
                   Summary,
                   comm_class_exampleSource.DatasetSet,
                   comm_class.OutputSet,
                   BaseModel):
    ...        
    

def SummaryModelRunner(opts:SummaryModel):
    comment = """
说明：
- 括号里面的值为与上次结果相比增加的百分比
- cg/dav使用PW测试集，elpa/scalapack使用LCAO测试集
- omp指计算命令为OMP_NUM_THREADS=16 mpirun -np 1 abacus
- 不含omp指计算命令为OMP_NUM_THREADS=1 mpirun -np 16 abacus
- SCFConverge Score以50步作为100分，值越大，表明收敛需要的步数越少
- Performance Score以abacus v3.0.0 的结果为基准，值越大表明总耗时越少，计算效率越高：
    - cg/dav以v3.0.0 intel-cg的结果为100分
    - genelpa/scalapack以v3.0.0 intel-elpa的结果为100分   
    """
    setting = {
        "experiment": "abacustest/benchmark",
        "experiment_id": "7ab4e46a-43fb-440a-828d-4fbdef5b4709",
        "profile": [{"title": "PW Performance Test",
                     "profiles": ["intel-cg","intel-dav","gnu-dav","omp-intel-dav",]},
                    {"title": "LCAO Performance Test",
                     "profiles": ["intel-elpa","intel-scalapack","gnu-elpa","omp-intel-elpa",]},
                    {"title": "Function Test",
                     "profiles": ["exx-test","examples",]}
        ],
        "aim_tag": {
            "intel-cg": ["benchmark-profile-pw-cg", "benchmark-schedule-intel-cg"],
            "intel-dav": ["benchmark-profile-pw-dav", "benchmark-schedule-intel-dav"],
            "intel-elpa": ["benchmark-profile-lcao-genelpa", "benchmark-schedule-intel-elpa"],
            "intel-scalapack": ["benchmark-profile-lcao-scalapack", "benchmark-schedule-intel-scalapack"],
            "gnu-cg": ["benchmark-profile-abacus-gnu-pw-cg", "benchmark-schedule-gnu-cg"],
            "gnu-dav": ["benchmark-profile-abacus-gnu-pw-david", "benchmark-schedule-gnu-dav"],
            "gnu-elpa": ["benchmark-profile-gnu-lcao-elpa", "benchmark-schedule-gnu-elpa"],
            "gnu-scalapack": ["benchmark-profile-gnu-lcao-scalapack", "benchmark-schedule-gnu-scalapack"],
            "omp-intel-cg": ["benchmark-profile-OMP-intel-cg", "benchmark-schedule-omp-intel-cg"],
            "omp-intel-dav": ["benchmark-profile-OMP-intel-dav", "benchmark-schedule-omp-intel-dav"],
            "omp-intel-elpa": ["benchmark-profile-OMP-intel-elpa", "benchmark-schedule-omp-intel-elpa"],
            "omp-intel-scalapack": ["benchmark-profile-OMP-intel-scalapack", "benchmark-schedule-omp-intel-scalapack"],
            "omp-gnu-cg": ["benchmark-profile-OMP-gnu-cg", "benchmark-schedule-omp-gnu-cg"],
            "omp-gnu-dav": ["benchmark-profile-OMP-gnu-dav", "benchmark-schedule-omp-gnu-dav"],
            "omp-gnu-elpa": ["benchmark-profile-OMP-gnu-elpa", "benchmark-schedule-omp-gnu-elpa"],
            "omp-gnu-scalapack": ["benchmark-profile-OMP-gnu-scalapack", "benchmark-schedule-omp-gnu-scalapack"],
            "exx-test": ["benchmark-schedule-exx-test"],
            "examples":["benchmark-schedule-exampletest"]
        },
        "metrics": ["NormalEnd_ratio", "Converge_ratio", "SCFConverge Score", "Performance Score"],
        "metrics_name": {
            "NormalEnd_ratio": ["NormalEnd_ratio","TrueRatio(normal_end)","pass_ratio"],
            "Converge_ratio": ["converge_ratio","TrueRatio(converge)"],
            "SCFConverge Score": ["iGM(SCF_steps)","iGM(scf_steps)"],
            "Performance Score": ["iGM(total_time)"]
        },
        "value_range": {
            "NormalEnd_ratio": [0,1.2],
            "Converge_ratio": [0,1.2],
            "SCFConverge Score": [10,500],
            "Performance Score": [10,500]
        },
        "metrics_coef": {
            "NormalEnd_ratio": 1,
            "Converge_ratio": 1,
            "SCFConverge Score": 100*50,
            "Performance Score": {
                "intel-cg": 100 / 1.555e-03,
                "intel-dav": 100 / 1.555e-03,
                "intel-elpa": 100 / 2.605e-03,
                "intel-scalapack": 100 / 2.605e-03,
                "gnu-cg": 100 / 1.555e-03,
                "gnu-dav": 100 / 1.555e-03,
                "gnu-elpa": 100 / 2.605e-03,
                "gnu-scalapack": 100 / 2.605e-03,
                "omp-intel-cg": 100 / 1.555e-03,
                "omp-intel-dav": 100 / 1.555e-03,
                "omp-intel-elpa": 100 / 2.605e-03,
                "omp-intel-scalapack": 100 / 2.605e-03,
                "omp-gnu-cg": 100 / 1.555e-03,
                "omp-gnu-dav": 100 / 1.555e-03,
                "omp-gnu-elpa": 100 / 2.605e-03,
                "omp-gnu-scalapack": 100 / 2.605e-03
            }
        },
        "digit": {
            "NormalEnd_ratio": 2,
            "Converge_ratio": 2,
            "SCFConverge Score": 0,
            "Performance Score": 0
        },
        "comment": comment,
        "chart_setting":[
            {
                "profiles": ["intel-cg","intel-dav","gnu-dav","omp-intel-dav"],
                "metrics": ["NormalEnd_ratio"],
                "title": "PW NormalEnd_ratio"
            },
            {
                "profiles": ["intel-cg","intel-dav","gnu-dav","omp-intel-dav"],
                "metrics": ["Converge_ratio"],
                "title": "PW Converge_ratio"
            },
            {
                "profiles": ["intel-cg","intel-dav","gnu-dav","omp-intel-dav"],
                "metrics": ["SCFConverge Score"],
                "title": "PW SCFConverge Score"
            },
            {
                "profiles": ["intel-cg","intel-dav","gnu-dav","omp-intel-dav"],
                "metrics": ["Performance Score"],
                "title": "PW Performance Score"
            },
            {
                "profiles": ["intel-elpa","intel-scalapack","gnu-elpa","omp-intel-elpa"],
                "metrics": ["NormalEnd_ratio"],
                "title": "LCAO NormalEnd_ratio"
            },
            {
                "profiles": ["intel-elpa","intel-scalapack","gnu-elpa","omp-intel-elpa"],
                "metrics": ["Converge_ratio"],
                "title": "LCAO Converge_ratio"
            },
            {
                "profiles": ["intel-elpa","intel-scalapack","gnu-elpa","omp-intel-elpa"],
                "metrics": ["SCFConverge Score"],
                "title": "LCAO SCFConverge Score"
            },
            {
                "profiles": ["intel-elpa","intel-scalapack","gnu-elpa","omp-intel-elpa"],
                "metrics": ["Performance Score"],
                "title": "LCAO Performance Score"
            },
            {
                "profiles": ["exx-test","examples"],
                "metrics": ["NormalEnd_ratio"],
                "title": "Other functions test"
            }
        ]
    }
    output_path = opts.IO_output_path
    os.makedirs(output_path,exist_ok=True)
    table_html_file_name = "result.html"
    table_html_file = os.path.join(output_path,table_html_file_name)
    chart_html_file_name = "all.html"
    chart_html_file = os.path.join(output_path,chart_html_file_name)
    
    token = opts.AIM_TOKEN
    if opts.setting != None and opts.setting.strip() != "":
        setting = json.loads(opts.setting)
    elif opts.setting_file != None:
        setting = json.load(open(opts.setting_file.get_full_path(),'r'))
    elif opts.dataset != None and opts.setting_file_name != None and opts.setting_file_name.strip() != "":
        dataset_work_path = comm_class_exampleSource.get_dataset_work_path(opts)
        try:
            setting = json.load(open(os.path.join(dataset_work_path,opts.setting_file_name),'r'))
        except:
            traceback.print_exc()
            return 1
    experiment = setting.get("experiment","abacustest/benchmark")
    experiment_id = setting.get("experiment_id","7ab4e46a-43fb-440a-828d-4fbdef5b4709")
    profile = setting.get("profile",[])
    aim_tag = setting.get("aim_tag",{})
    metrics = setting.get("metrics",[])
    metrics_name = setting.get("metrics_name",{})
    metrics_coef = setting.get("metrics_coef",{})
    digit = setting.get("digit",{})
    comment = setting.get("comment","")
    value_range = setting.get("value_range",{})
    chart_setting = setting.get("chart_setting",[])
    remote_setting = setting.get("remote",{})
    metrics_color = setting.get("metrics_color",{})
    color_list = setting.get("color_list",["green","red"])
    
    json.dump(setting,open(os.path.join(output_path,"setting.json"),'w'),indent=4)

    if not opts.if_load_data:
        allprofiles = []
        for ipro in profile: allprofiles += ipro.get("profiles",[])
        
        alltags = []
        for i in allprofiles:
            alltags += aim_tag.get(i,[i])
        allruns,allruninfos = get_aim_data.get_runs(token,experiment,experiment_id,alltags,collect_metrics=False,GetVersion=False)
        allvalues = get_profile_value(allruninfos,allprofiles,aim_tag,metrics,metrics_coef,metrics_name,token)
        pickle.dump(allvalues,open("tracking.pkl",'wb'))
    else:
        allvalues = pickle.load(open("tracking.pkl",'rb'))
    
    #send to feishu and produce html file
    outtable = produce_outtable(allvalues,profile,metrics,digit,metrics_color,color_list)
    if opts.feishu_webhook:
        current_job = opts.Config_dflow_labels["launching-job"]
        current_url = "https://app.bohrium.dp.tech/abacustest/?request=GET%3A%2Fapplications%2Fabacustest%2Fjobs%2F" + current_job
        comment_add = f"\nClick profile to check the detail Benchmark run.\n\nClick [here]({current_url}) to view detailed reports and historical trend graphs."
        send_to_feishu(outtable,opts.feishu_webhook,comment+comment_add)
    html_content,html_content_nolink = produce_html(outtable,comment)
    with open(table_html_file,'w') as f1: f1.write(html_content)
    with open(table_html_file.replace(".html","_nolink.html"),'w') as f1: f1.write(html_content_nolink)

    #plot
    html_section = ReportSection(title="metrics chart",
                             elements=[AutoReportElement(title='metrics', path=table_html_file_name, description="")])
    #chart_section = echart_report(allvalues)
    echart_html(allvalues,value_range,chart_setting,chart_html_file)
    echart_html_section = ReportSection(title="metrics chart",
                             elements=[AutoReportElement(title='metrics', path=chart_html_file_name, description="")])

    report = Report(title="abacus test report",
                        sections=[html_section,echart_html_section],
                        description="a report of abacustest") 
    report.save(output_path)
    
    pwd = os.getcwd()
    os.chdir(output_path)
    if remote_setting:
        from abacustest import remote
        if "github" in remote_setting and "token_file" in remote_setting["github"] and remote_setting["github"]["token_file"].strip() != "":
            remote_setting["github"]["token"] = open(remote_setting["github"]["token_file"]).read().strip()
        remote_path = remote.prepare_files(remote_setting)
        if remote_path:
            remote.push_to_remote(remote_path,remote_setting,{"github_username": getattr(opts,"Config_github_username"),
                                                              "github_email": getattr(opts,"Config_github_email"),
                                                              "github_token": getattr(opts,"Config_github_token")})
    os.chdir(pwd)
    #parse inputs  
    return 0

'''


'''

